//--------------------------------------------------------------------------
//
//			by Serge Fantino
//			fantino@math.unice.fr
//
//		Copyright 1998,1999, All Rights Reserved.
//
//--------------------------------------------------------------------------

#ifndef PACK_VIEW_H
#define PACK_VIEW_H

//#include "myBox.h"
#include <InterfaceKit.h>

#include "spliter.h"
#include "interface_kit_tools.h"

#ifndef PACK_VIEW_CONSTANT
#define PACK_VIEW_CONSTANT

enum {
	K_NONE =			0,
	// ---
	K_WARP_TOP =		1,
	K_WARP_BOTTOM =		2,
	// ---
	K_WARP_LEFT =		4,
	K_WARP_RIGHT =		8,
	// ---
	K_WARP_ALL =		16,
	// ---
	K_FILL_X =			32,
	K_FILL_Y =			64,
};

enum {
	K_TOP =			1,
	K_BOTTOM =		2,
	K_LEFT =		4,
	K_RIGHT =		8,
	K_FULL =                          16,
};

const uint32 K_FILL_BOTH = K_FILL_X | K_FILL_Y;
const uint32 K_FILL_NONE = K_NONE;

#endif

#define PACK_INFO
/* store widget placement info */
struct pack_info {

  pack_info(){};

  pack_info(BView *w,uint32 s,uint32 a,uint32 f,bool x,float hb,float vb
	    )
    {
      view=w;side=s;anchor=a;fill=f;expand=x;hborder=hb;vborder=vb;
    };
  
  // "override" widget bounds width&height to take border in account
  float Width() {return view->Bounds().Width()+hborder+hborder;};
  float Height() {return view->Bounds().Height()+vborder+vborder;};

  BView *view;// the widget
  uint32 side;// the side where to allocate the widget parcel 
  uint32 anchor;// where to anchor the widget in its parcel
  uint32 fill;// how the widget should fill its parcel
  bool expand;// deprecated ?
  float hborder,vborder;// horizontal & vertical border to add around the widget
}; 

enum split_mode {
	K_NO_SEPARATOR,
	K_SEPARATOR,
	K_SPLITER
};

class KPackView;

class KPack {
public:
	KPack(BView *view,
		uint32 side=K_TOP,
		uint32 anchor=K_TOP | K_LEFT,
		uint32 fill=K_FILL_NONE,
		BPoint border=BPoint(0,0)
		);
		
	KPack(struct pack_info info);
		
private:
	friend KPackView;

	struct pack_info fInfo;
};

BRect compute_free_parcel(BRect frame,pack_info *info,BRect *pt_view_parcel=NULL);

class KPackView : public BBox, public KExtendable {
public:
	KPackView(BRect frame, const char *name, 
		uint32 resizingMode=B_FOLLOW_ALL_SIDES, 
		uint32 flags=0,
		border_style border = B_NO_BORDER);
		
	KPackView(const char *name,KPack **pack_list);
	
	void Init();
		
	virtual ~KPackView();
	
	KPackView *Split(split_mode mode=K_SPLITER,BView *spliter=NULL);
	void SetPosture(orientation posture);
	
	virtual void PackChild(BView *view,
		uint32 side,
		uint32 anchor,
		uint32 fill,
		//bool expand=false,
		BPoint border,
		pack_info **pack=NULL
		);
		
	// for backward compatibility...
	
	void PackChild(BView *view,
		uint32 side,
		uint32 anchor,
		uint32 fill,
		bool expand,
		BPoint border=BPoint(0,0)
		)
	{
		PackChild(view,(expand==true)?K_FULL:side,anchor,fill,border);
	}
	
		
	// because of overloading...
	void PackChild(BView *view,
		uint32 side=K_TOP,
		uint32 anchor=K_TOP | K_LEFT,
		uint32 fill=K_FILL_NONE
		)
	{
		PackChild(view,side,anchor,fill,BPoint(0,0));
	}
		
	void PackChild(KPack *pack);
	void PackChild(KPack **pack_list);
		
	void AttachedToWindow(void); 
	virtual void AllAttached(void);
	
	virtual void GetPreferredSize(float *width,float *height);
	
	// this one is called only ONE time, just after the view was attached for the first time
	// rem: it's a good place to setup child view, because the view already belongs to a looper,
	//      so you can setup target to this safely ...
	virtual void ReadyToPrepare();
	virtual void AttachedToWindow(bool first_time); 
	
	//virtual void FrameResized(float width, float height);
	
	virtual void ExtendBy(float x,float y,bool lock_size_limit=true,BView *source=NULL);
	//virtual void ExtendBy(float x,float y);
	
protected:

	void PackQueued();
	void RePack();

	BList	*fPackQueue,*fPackStack;
	
private:
/*
	virtual void really_pack_child(BView *view,
		uint32 side=K_TOP,
		uint32 anchor=K_TOP | K_LEFT,
		uint32 fill=K_FILL_NONE,
		bool expand=false,
		float border=0
		);
*/
	virtual void really_pack_child(pack_info *info);
		
	BRect ComputeFreeParcel();
	
private:
	typedef BBox inherited;

	bool	fPropagate;
	bool	fAttachedOnce;
	
	uint32	fOrientation;
	

	BRect	fFree,fBounds;
	BPoint	fFixedPreferredSize,fPreferredSize;
	bool	fFull;
	int		fBorder;
	
	KPackView *fActualPacker;
	KSplitView *fSeparator;
	struct pack_info *fLastPacked;
};

class KDualPaneView : public KPackView {
public:

	KDualPaneView(BRect frame,const char *name,BView *first,BView *second,uint32 where,
		uint32 resizingMode=B_FOLLOW_ALL_SIDES, 
		uint32 flags=0,
		border_style border = B_NO_BORDER);
	
	virtual void ExtendBy(float x,float y,bool lock_size_limit=false,BView *source=NULL);

	void SetDirection(uint32 direction);
	
	virtual void MessageReceived(BMessage *message);

private:

	KSplitView	*fSplit;
	pack_info 	*fFirst,*fSecond,*fSeparator;
};

#endif
