//--------------------------------------------------------------------------
//
//			by Serge Fantino
//			fantino@math.unice.fr
//
//		Copyright 1998-1999, All Rights Reserved.
//
//--------------------------------------------------------------------------

#include <stdio.h>
#include <Debug.h>

#include "pack_view.h"
//#include "interface_kit_tools.h"

#include <SupportKit.h>

/* compute widget parcel :
   - given the available frame and the widget pack_info structure,
   - compute the parcel allocated to the widget (pt_view_parcel) and the resulting free parcel
*/
/* NB: the widget parcel frame relies on the current widget size... this one may (and in general will) be different if the widget is attached or not */
BRect compute_free_parcel(BRect frame,pack_info *info,BRect *pt_view_parcel)
{
  BRect view_parcel=frame;
  BRect free_parcel=frame;
  if (info->side==K_RIGHT) {
    view_parcel.left=frame.right-info->Width();
    free_parcel.right=view_parcel.left-1;
  } else if (info->side==K_LEFT) {
    view_parcel.right=frame.left+info->Width();
    free_parcel.left=view_parcel.right+1;
  } else if (info->side==K_BOTTOM) {
    view_parcel.top=frame.bottom-info->Height();
    free_parcel.bottom=view_parcel.top-1;
  } else {// info->side==K_TOP
    view_parcel.bottom=frame.top+info->Height();
    free_parcel.top=view_parcel.bottom+1;
  }
  if (pt_view_parcel!=NULL)
    *pt_view_parcel=view_parcel;
  return free_parcel;
};

KPack::KPack(BView *view,
		uint32 side,
		uint32 anchor,
		uint32 fill,
		BPoint border
		)
{
	fInfo=pack_info(view,side,anchor,fill,side==K_FULL,border.x,border.y);
}
		
KPack::KPack(struct pack_info info)
{
	fInfo=info;
}
		

// KPackView <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

KPackView::KPackView(BRect frame, const char *name, 
	uint32 resizingMode, uint32 flags,
	border_style border):
inherited(frame,name,resizingMode,flags | B_FRAME_EVENTS | B_WILL_DRAW,border)
{
	Init();
}

KPackView::KPackView(const char *name,KPack **pack_list):
inherited(BRect(0,0,1,1),name,B_FOLLOW_ALL,B_FRAME_EVENTS | B_WILL_DRAW,B_NO_BORDER)
{
	Init();
	// ---
	while (*pack_list!=NULL) {
		PackChild(*pack_list);
		++pack_list;
	}
}

void KPackView::Init()
{
	fActualPacker=this;// new !!!
	fSeparator=NULL;
  	fLastPacked=NULL;
	fPropagate=true;
	fAttachedOnce=false;
	fFull=false;
	fPackQueue=new BList();
	fPackStack=new BList();
	// ---
	fPreferredSize=Bounds().RightBottom();
	fFixedPreferredSize=BPoint(0,0);
}

KPackView::~KPackView()
{
  for (int32 i=fPackQueue->CountItems();i>0;i--)
    delete fPackQueue->RemoveItem((int32)0);
  for (int32 i=fPackStack->CountItems();i>0;i--)
    delete fPackStack->RemoveItem((int32)0);
  delete fPackQueue;
  delete fPackStack;
}

KPackView *KPackView::Split(split_mode mode,BView *spliter)
{
	if (fActualPacker!=this) return fActualPacker->Split(mode,spliter);
	if (fLastPacked==NULL||fLastPacked->side==K_FULL) return NULL;
	orientation posture=B_VERTICAL;
	uint32 fill=K_FILL_Y;
	if (fLastPacked->side==K_TOP||fLastPacked->side==K_BOTTOM) {
		posture=B_HORIZONTAL;
		fill=K_FILL_X;
	}
	if (spliter!=NULL) {
		PackChild(spliter,fLastPacked->side,fLastPacked->anchor,fill);
	} else switch (mode) {
		case K_NO_SEPARATOR:
			break;
		case K_SEPARATOR:
			PackChild(new KSeparatorView(posture,"split"),fLastPacked->side,fLastPacked->anchor,fill);
			break;
		case K_SPLITER:
			fSeparator=new KSplitView(posture,"split");
			PackChild(fSeparator,fLastPacked->side,fLastPacked->anchor,fill);
			/*
			if (posture==B_HORIZONTAL) {
				split->SetLimits(fLastPacked->Height(),0);
			} else {
				split->SetLimits(fLastPacked->Width(),0);
			}
			*/
			break;
	}
	KPackView *pack=new KPackView(BRect(0,0,1,1),"sub_pack",B_FOLLOW_ALL,B_FRAME_EVENTS | B_WILL_DRAW,B_NO_BORDER);
	PackChild(pack,K_FULL,fLastPacked->anchor,K_FILL_BOTH);
	fActualPacker=pack;
	return pack;
}

// extend the pack view by x/y
// NB : should be called only if the pack view is attached to a window
void KPackView::ExtendBy(float x,float y,bool lock_size_limit,BView *source)
{
	extend_view_by(this,x,y,lock_size_limit,this);
}
/*
void KPackView::ExtendBy(float x,float y)
{
	extend_view_by(this,x,y,true);
}
*/	
// the replacement for the AddChild - 
/* */
void KPackView::PackChild(BView *view,uint32 side,
			  uint32 anchor,
			  uint32 fill,
			  //bool expand,
			  BPoint border,
			  pack_info  **pack
			  )
{
  if (fActualPacker!=this) {
  	fActualPacker->PackChild(view,side,anchor,fill,border,pack);
  } else {
 	pack_info *info=new pack_info(view,side,anchor,fill,true,border.x,border.y);
 	if (pack!=NULL) *pack=info;
 	fLastPacked=info;
	if (Window()!=NULL) {// add the widget NOW !
  	  really_pack_child(info);
  	  //delete info;
  	} else {// delay adding widget until the Packer is attached
  	  fPackQueue->AddItem(info);
  	}
  }
}

void KPackView::PackChild(KPack *pack)
{
  if (fActualPacker!=this) {
  	fActualPacker->PackChild(pack);
  } else {
  	pack_info *info=new pack_info(pack->fInfo);
  	fLastPacked=info;
  	if (Window()!=NULL) {// add the widget NOW !
  	  really_pack_child(info);
  	  //delete info;
  	} else {// delay adding widget until the Packer is attached
  	  fPackQueue->AddItem(info);
  	}
  }
}

void KPackView::PackChild(KPack **pack_list)
{
	while (*pack_list!=NULL) {
		PackChild(*pack_list);
		++pack_list;
	} 
}
	

// compute the free parcel of the pack view
/* - this is a little bit time consuming, since we go througth all the packed widgets,
      but it's safe, cause we use actual widgets bounds...
*/
BRect KPackView::ComputeFreeParcel()
{
  BRect full=Bounds();
  // take care of border and label 
  // - THIS IS BAD cause we need to hard-code it...
  // - if someone knows how I can rely on actual values...
  if (Border()!=B_NO_BORDER)
    full.InsetBy(5,5);
  if (Label()!=NULL)
    full.top+=10;
  // go througth the list
  for (int32 i=fPackStack->CountItems()-1;i>=0;i--) {
    pack_info *pack=(pack_info *)fPackStack->ItemAt(i);
    full=compute_free_parcel(full,pack);
  }
  return full;
} 
	
// really pack a widget
/* NB : when we get here, we are sure that the pack view is attached to a window;
          this is important cause we could do some resizing on the widget,
          and for this to perform right, it need to be attached either...
*/
void KPackView::really_pack_child(pack_info *info)
{
  //PRINT(("%s is packing %s\n",Name(),info->view->Name()));

  uint32 resizingMode=0;
  float add_X=0,add_Y=0;

  // ------ add view
  // neutralize resizing for now... we will setup that latter depending on packing info
  // it's use by the extend mechanism to correctly handle packing order
  info->view->SetResizingMode(0);
  // ok, pack
  PRINT(("%s ADDCHILD %s\n",Name(),info->view->Name()));
  AddChild(info->view);// first, add the widget, to be sure we get its actual size, and we can resize it nicely
  PRINT(("%s ADDED %fx%f\n",info->view->Name(),info->view->Bounds().Width(),info->view->Bounds().Height()));
  float width,height;
  info->view->GetPreferredSize(&width,&height);
  PRINT(("%s preferred size is %fx%f\n",info->view->Name(),width,height));
  info->view->ResizeTo(width,height);
  //PRINT(("---\n");
  // ------- compute free parcel
  fFree=ComputeFreeParcel();
  BRect view_parcel=fFree,free_parcel=fFree;

  // ------- add to pack-stack
  //fPackStack->AddItem(new pack_info(*info),0);
  //fPackStack->AddItem(info,0);
  fPackStack->AddItem(info);

  // ------- define view_parcel & define resizing mode
  if (info->side==K_FULL) {
    // use all the available free parcel
    fFixedPreferredSize+=BPoint(width,height);
    view_parcel=free_parcel;
    free_parcel.top=view_parcel.bottom;
    add_Y=info->Height()-view_parcel.Height();
    add_X=info->Width()-view_parcel.Width();
    fFull=true;
  } else if (info->side==K_RIGHT) {
    // pack the widget at right in the free parcel
    fFixedPreferredSize+=BPoint(width,0);
    fPreferredSize.y=max_c(fPreferredSize.y,height);
    view_parcel.left=fFree.right-info->Width();
    free_parcel.right=view_parcel.left-1;
    add_Y=info->Height()-view_parcel.Height();
    resizingMode=resizingMode|B_FOLLOW_RIGHT;
  } else if (info->side==K_LEFT) {
    // pack the widget at left in the free parcel
    fFixedPreferredSize+=BPoint(width,0);
    fPreferredSize.y=max_c(fPreferredSize.y,height);
    view_parcel.right=fFree.left+info->Width();
    free_parcel.left=view_parcel.right+1;
    add_Y=info->Height()-view_parcel.Height();
    resizingMode=resizingMode|B_FOLLOW_LEFT;
  } else if (info->side==K_BOTTOM) {
    // pack the widget at bottom in the free parcel
    fFixedPreferredSize+=BPoint(0,height);
    fPreferredSize.x=max_c(fPreferredSize.x,width);
    view_parcel.top=fFree.bottom-info->Height();
    free_parcel.bottom=view_parcel.top-1;
    add_X=info->Width()-view_parcel.Width();
    resizingMode=resizingMode|B_FOLLOW_BOTTOM;
  } else {
    // pack the widget at top in the free parcel
    fFixedPreferredSize+=BPoint(0,height);
    fPreferredSize.x=max_c(fPreferredSize.x,width);
    view_parcel.bottom=fFree.top+info->Height();
    free_parcel.top=view_parcel.bottom+1;
    add_X=info->Width()-view_parcel.Width();
    resizingMode=resizingMode|B_FOLLOW_TOP;
  }
  fFree=free_parcel;

  // ------ take care of borders
  view_parcel.InsetBy(info->hborder,info->vborder);

  // ------ anchor
  BRect real_parcel=view_parcel;
  if ((info->side==K_LEFT||info->side==K_RIGHT)&&!(info->fill&K_FILL_Y)) {
    if (info->anchor&K_BOTTOM) {
      real_parcel.top=real_parcel.bottom-info->view->Bounds().Height();
      resizingMode=resizingMode|B_FOLLOW_BOTTOM;
    } else
      resizingMode=resizingMode|B_FOLLOW_TOP;
  } else if ((info->side==K_TOP||info->side==K_BOTTOM)&&!(info->fill&K_FILL_X)) {
    if (info->anchor&K_RIGHT) {
      real_parcel.left=real_parcel.right-info->view->Bounds().Width();
      resizingMode=resizingMode|B_FOLLOW_RIGHT;
    } else 
      resizingMode=resizingMode|B_FOLLOW_LEFT;
  }
  
  // ------ fill ?
  if (info->fill&K_FILL_X) {
    info->view->ResizeTo(view_parcel.Width(),info->view->Bounds().Height());
    resizingMode=resizingMode|B_FOLLOW_LEFT_RIGHT;
  }
  if (info->fill&K_FILL_Y) {
    info->view->ResizeTo(info->view->Bounds().Width(),view_parcel.Height());
    resizingMode=resizingMode|B_FOLLOW_TOP_BOTTOM;
  }

  // ------ reshape the widget
  info->view->MoveTo(real_parcel.left,real_parcel.top);
  info->view->SetResizingMode(resizingMode);

//PRINT(("%s is %f0x%f0\n",info->view->Name(),info->view->Bounds().Width(),info->view->Bounds().Height()));
  // ------ augment view_parcel ?
  if (add_X>0) {
    ExtendBy(add_X,0);
    view_parcel.right=view_parcel.right+add_X;
    fFree.right=fFree.right+add_X;
  }
  if (add_Y>0) {
    ExtendBy(0,add_Y);
    view_parcel.bottom=view_parcel.bottom+add_Y;
    fFree.bottom=fFree.bottom+add_Y;
  }
  
  if (fFree.Height()<0) {
    ExtendBy(0,-fFree.Height()-1);
    fFree.bottom=fFree.top+3;
  }
  if (fFree.Width()<0) {
    ExtendBy(-fFree.Width()-1,0);
    fFree.right=fFree.left+3;
  }
  PRINT(("%s size after packing %s is %fx%f\n",Name(),info->view->Name(),Bounds().Width(),Bounds().Height()));
}

void KPackView::PackQueued()
{
	Window()->DisableUpdates();
	for (int32 i=fPackQueue->CountItems();i>0;i--) {
		pack_info *args=(pack_info *)fPackQueue->ItemAt(0);
		//really_pack_child(args->view,args->side,args->anchor,args->fill,args->expand);
		really_pack_child(args);
		fPackQueue->RemoveItem((int32)0);
	}
	Window()->EnableUpdates();
}

void KPackView::RePack()
{
	PackQueued();
	BList *temp=fPackQueue;
	fPackQueue=fPackStack;
	fPackStack=temp;
	//
	for (int32 i=CountChildren();i>0;i--) 
		ChildAt(0)->RemoveSelf();
	//
	fPreferredSize=BPoint(0,0);
	fFixedPreferredSize=BPoint(0,0);
	PackQueued();
}

void KPackView::AttachedToWindow(void)
{
	//PRINT(("%s was attached\n",Name()));
	
  PRINT(("BEGIN ATTACHED %s\n",Name()));
  inherited::AttachedToWindow();
  // ---------
  if (Parent()!=NULL)
    SetViewColor(Parent()->ViewColor()); 
  //fFree=Bounds();
  // -----------
  fPreferredSize=BPoint(0,0);
  fFixedPreferredSize=BPoint(0,0);
  PackQueued();
  // -----------
  if (!fAttachedOnce) ReadyToPrepare();
  AttachedToWindow(!fAttachedOnce);
  //if (!fAttachedOnce) {
    fAttachedOnce=true;
  //}
  if (fSeparator!=NULL) {
		fSeparator->SetDefaultLimits();
	}
  PRINT(("END ATTACHED %s \n",Name()));
}

void KPackView::AllAttached(void)
{
	PRINT(("ALLATTACHED"));
	inherited::AllAttached();
	if (fSeparator!=NULL) {
		fSeparator->SetDefaultLimits();
	}
}

void KPackView::ReadyToPrepare()
{
}

void KPackView::AttachedToWindow(bool first_time)
{
}

void KPackView::GetPreferredSize(float *width,float *height)
{
	// recalculate ???
	*width=fPreferredSize.x+fFixedPreferredSize.x;
	*height=fPreferredSize.y+fFixedPreferredSize.y;
	//inherited::GetPreferredSize(width,height);
}


// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< DUAL SPLIT PANE >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

KDualPaneView::KDualPaneView(BRect frame,const char *name,BView *first,BView *second,uint32 where,
		uint32 resizingMode, 
		uint32 flags,
		border_style border)
:KPackView(frame,name,resizingMode,flags,border)
{
	uint32 fill=K_FILL_X;
	if (where==K_LEFT||where==K_RIGHT) fill=K_FILL_Y;
	fSplit=new KSplitView(((fill==K_FILL_X)?B_HORIZONTAL:B_VERTICAL),"_split_");
	// ---
	BPoint border(0,0);
	//if (fSplit->Posture()==B_HORIZONTAL) border=BPoint(fSplit->Width(),0);
	//else border=BPoint(0,fSplit->Width());
	// ---
	PackChild(first,where,K_LEFT,fill,border,&fFirst);
	PackChild(fSplit,where,K_LEFT,fill,BPoint(0,0),&fSeparator);
	PackChild(second,K_FULL,K_TOP,K_FILL_BOTH,border,&fSecond);
}
	
void KDualPaneView::ExtendBy(float x,float y,bool lock_size_limit,BView *source)
{
	//PRINT(("IL PASSE ICI !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"));
	if (source==fFirst->view) {
		if (fSplit->Posture()==B_HORIZONTAL&&y>0) {
			fSplit->SplitTo(fFirst->view->Frame().RightBottom()+BPoint(0,y));
			if (lock_size_limit) fSplit->SetLimits(fFirst->view->Bounds().Height(),0);
			// ---
			y=0;
		} else if (x>0) {
			fSplit->SplitTo(fFirst->view->Frame().RightBottom()+BPoint(x,0));
			if (lock_size_limit) fSplit->SetLimits(fFirst->view->Bounds().Width(),0);
			// ---
			x=0;
		}
	}
	KPackView::ExtendBy(x,y,lock_size_limit,this);
}

void KDualPaneView::SetDirection(uint32 where)
{
	uint32 fill=K_FILL_X;
	if (where==K_LEFT||where==K_RIGHT) fill=K_FILL_Y;
	if (fill==K_FILL_X) fSplit->SetPosture(B_HORIZONTAL);
	else fSplit->SetPosture(B_VERTICAL);
	//BPoint border;
	//if (fSplit->Posture()==B_HORIZONTAL) border=BPoint(fSplit->Width(),0);
	//else border=BPoint(0,fSplit->Width());
	// ---
	fFirst->side=where;
	fFirst->fill=fill;
	//fFirst->hborder=border.x;
	//fFirst->vborder=border.y;
	// ---
	fSeparator->side=where;
	fSeparator->fill=fill;
	fSplit->ResizeTo(fSplit->Width(),fSplit->Width());// ARG
	// ---
	//fSecond->hborder=border.x;
	//fSecond->vborder=border.y;
	// ---
	RePack();
}

void KDualPaneView::MessageReceived(BMessage *message)
{
	switch (message->what) {
		case K_SET_HORIZONTAL:
			SetDirection(K_TOP);
			break;
		case K_SET_VERTICAL:
			SetDirection(K_LEFT);
			break;
		default:
			KPackView::MessageReceived(message);
	}
}



