//--------------------------------------------------------------------------
//
//			by Serge Fantino
//			fantino@math.unice.fr
//
//		Copyright 1998-1999, All Rights Reserved.
//
//--------------------------------------------------------------------------

#include <stdio.h>

#include "pack_view.h"

#include <SupportKit.h>

/* compute widget parcel :
   - given the available frame and the widget pack_info structure,
   - compute the parcel allocated to the widget (pt_view_parcel) and the resulting free parcel
*/
/* NB: the widget parcel frame relies on the current widget size... this one may (and in general will) be different if the widget is attached or not */
BRect compute_free_parcel(BRect frame,pack_info *info,BRect *pt_view_parcel=NULL)
{
  BRect view_parcel=frame;
  BRect free_parcel=frame;
  if (info->side==K_RIGHT) {
    view_parcel.left=frame.right-info->Width();
    free_parcel.right=view_parcel.left-1;
  } else if (info->side==K_LEFT) {
    view_parcel.right=frame.left+info->Width();
    free_parcel.left=view_parcel.right+1;
  } else if (info->side==K_BOTTOM) {
    view_parcel.top=frame.bottom-info->Height();
    free_parcel.bottom=view_parcel.top-1;
  } else {// info->side==K_TOP
    view_parcel.bottom=frame.top+info->Height();
    free_parcel.top=view_parcel.bottom+1;
  }
  if (pt_view_parcel!=NULL)
    *pt_view_parcel=view_parcel;
  return free_parcel;
};

KPack::KPack(BView *view,
		uint32 side,
		uint32 anchor,
		uint32 fill,
		BPoint border
		)
{
	fInfo=pack_info(view,side,anchor,fill,side==K_FULL,border.x,border.y);
}
		
KPack::KPack(struct pack_info info)
{
	fInfo=info;
}
		

// KPackView <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

KPackView::KPackView(BRect frame, const char *name, 
	uint32 resizingMode, uint32 flags,
	border_style border):
inherited(frame,name,resizingMode,flags | B_FRAME_EVENTS | B_WILL_DRAW,border)
{
	Init();
}

KPackView::KPackView(const char *name,KPack **pack_list):
inherited(BRect(0,0,1,1),name,B_FOLLOW_ALL,B_FRAME_EVENTS | B_WILL_DRAW,B_NO_BORDER)
{
	Init();
	// ---
	while (*pack_list!=NULL) {
		PackChild(*pack_list);
		++pack_list;
	}
}

void KPackView::Init()
{
	fActualPacker=this;// new !!!
  	fLastPacked=NULL;
	fPropagate=true;
	fAttachedOnce=false;
	fFull=false;
	fPackQueue=new BList();
	fPackStack=new BList();
}

KPackView::~KPackView()
{
  for (int32 i=fPackQueue->CountItems();i>0;i--)
    delete fPackQueue->RemoveItem((int32)0);
  for (int32 i=fPackStack->CountItems();i>0;i--)
    delete fPackStack->RemoveItem((int32)0);
  delete fPackQueue;
  delete fPackStack;
}

KPackView *KPackView::Split(split_mode mode,BView *spliter)
{
	if (fActualPacker!=this) return fActualPacker->Split(mode);
	if (fLastPacked==NULL||fLastPacked->side==K_FULL) return NULL;
	orientation posture=B_VERTICAL;
	uint32 fill=K_FILL_Y;
	if (fLastPacked->side==K_TOP||fLastPacked->side==K_BOTTOM) {
		posture=B_HORIZONTAL;
		fill=K_FILL_X;
	}
	if (spliter!=NULL) {
		PackChild(spliter,fLastPacked->side,fLastPacked->anchor,fill);
	} else switch (mode) {
		case K_NO_SEPARATOR:
			break;
		case K_SEPARATOR:
			PackChild(new KSeparator(5,posture,"split"),fLastPacked->side,fLastPacked->anchor,fill);
			break;
		case K_SPLITER:
			PackChild(new KSplit(5,posture,"split"),fLastPacked->side,fLastPacked->anchor,fill);
			break;
	}
	KPackView *pack=new KPackView(BRect(0,0,1,1),"sub_pack",B_FOLLOW_ALL,B_FRAME_EVENTS | B_WILL_DRAW,B_NO_BORDER);
	PackChild(pack,K_FULL,fLastPacked->anchor,K_FILL_BOTH);
	fActualPacker=pack;
	return pack;
}

// extend the pack view by x/y
// NB : should be called only if the pack view is attached to a window
void KPackView::ExtendBy(float x,float y)
{
	PRINT(("%s is resizing by %fx%f\n",Name(),x,y));
	BView *v=this;
	while (v!=NULL) {
		if (v->ResizingMode()==0) {// if we found a view with no resizing at all...
			// resize it...
			v->ResizeBy(x,y);
			// and expect that further resizement will be done by someone... (a packer !)
			return;
		}
		v=v->Parent();
	}
	// ok, if all the ascendent view are well initialized,
	// extend the window ! (if any...)
	if (Window()!=NULL) {
		Window()->ResizeBy(x,y);
		float min_width,max_width,min_height,max_height;
		Window()->GetSizeLimits(&min_width,&max_width,&min_height,&max_height);
		Window()->SetSizeLimits(
		Window()->Bounds().Width(),max_width,
		Window()->Bounds().Height(),max_height);
  	}
}
	
// the replacement for the AddChild - 
/* */
void KPackView::PackChild(BView *view,uint32 side,
			  uint32 anchor,
			  uint32 fill,
			  //bool expand,
			  BPoint border
			  )
{
  if (fActualPacker!=this) {
  	fActualPacker->PackChild(view,side,anchor,fill,border);
  } else {
 	pack_info *info=new pack_info(view,side,anchor,fill,true,border.x,border.y);
 	fLastPacked=info;
	if (Window()!=NULL) {// add the widget NOW !
  	  really_pack_child(info);
  	  //delete info;
  	} else {// delay adding widget until the Packer is attached
  	  fPackQueue->AddItem(info);
  	}
  }
}

void KPackView::PackChild(KPack *pack)
{
  if (fActualPacker!=this) {
  	fActualPacker->PackChild(pack);
  } else {
  	pack_info *info=new pack_info(pack->fInfo);
  	fLastPacked=info;
  	if (Window()!=NULL) {// add the widget NOW !
  	  really_pack_child(info);
  	  //delete info;
  	} else {// delay adding widget until the Packer is attached
  	  fPackQueue->AddItem(info);
  	}
  }
}

void KPackView::PackChild(KPack **pack_list)
{
	while (*pack_list!=NULL) {
		PackChild(*pack_list);
		++pack_list;
	} 
}
	

// compute the free parcel of the pack view
/* - this is a little bit time consuming, since we go througth all the packed widgets,
      but it's safe, cause we use actual widgets bounds...
*/
BRect KPackView::ComputeFreeParcel()
{
  BRect full=Bounds();
  // take care of border and label 
  // - THIS IS BAD cause we need to hard-code it...
  // - if someone knows how I can rely on actual values...
  if (Border()!=B_NO_BORDER)
    full.InsetBy(5,5);
  if (Label()!=NULL)
    full.top+=10;
  // go througth the list
  for (int32 i=fPackStack->CountItems()-1;i>=0;i--) {
    pack_info *pack=(pack_info *)fPackStack->ItemAt(i);
    full=compute_free_parcel(full,pack);
  }
  return full;
} 
	
// really pack a widget
/* NB : when we get here, we are sure that the pack view is attached to a window;
          this is important cause we could do some resizing on the widget,
          and for this to perform right, it need to be attached either...
*/
void KPackView::really_pack_child(pack_info *info)
{
  PRINT(("%s is packing %s\n",Name(),info->view->Name()));

  uint32 resizingMode=0;
  float add_X=0,add_Y=0;

  // ------ add view
  // neutralize resizing for now... we will setup that latter depending on packing info
  // it's use by the extend mechanism to correctly handle packing order
  info->view->SetResizingMode(0);
  // ok, pack
  AddChild(info->view);// first, add the widget, to be sure we get its actual size, and we can resize it nicely
  info->view->ResizeToPreferred();
  //PRINT(("---\n");
  // ------- compute free parcel
  fFree=ComputeFreeParcel();
  BRect view_parcel=fFree,free_parcel=fFree;

  // ------- add to pack-stack
  //fPackStack->AddItem(new pack_info(*info),0);
  fPackStack->AddItem(info,0);

  // ------- define view_parcel & define resizing mode
  if (info->side==K_FULL) {
    // use all the available free parcel
    view_parcel=free_parcel;
    free_parcel.top=view_parcel.bottom;
    add_Y=info->Height()-view_parcel.Height();
    add_X=info->Width()-view_parcel.Width();
    fFull=true;
  } else if (info->side==K_RIGHT) {
    // pack the widget at right in the free parcel
    view_parcel.left=fFree.right-info->Width();
    free_parcel.right=view_parcel.left-1;
    add_Y=info->Height()-view_parcel.Height();
    resizingMode=resizingMode|B_FOLLOW_RIGHT;
  } else if (info->side==K_LEFT) {
    // pack the widget at left in the free parcel
    view_parcel.right=fFree.left+info->Width();
    free_parcel.left=view_parcel.right+1;
    add_Y=info->Height()-view_parcel.Height();
    resizingMode=resizingMode|B_FOLLOW_LEFT;
  } else if (info->side==K_BOTTOM) {
    // pack the widget at bottom in the free parcel
    view_parcel.top=fFree.bottom-info->Height();
    free_parcel.bottom=view_parcel.top-1;
    add_X=info->Width()-view_parcel.Width();
    resizingMode=resizingMode|B_FOLLOW_BOTTOM;
  } else {
    // pack the widget at top in the free parcel
    view_parcel.bottom=fFree.top+info->Height();
    free_parcel.top=view_parcel.bottom+1;
    add_X=info->Width()-view_parcel.Width();
    resizingMode=resizingMode|B_FOLLOW_TOP;
  }
  fFree=free_parcel;

  // ------ take care of borders
  view_parcel.InsetBy(info->hborder,info->vborder);

  // ------ anchor
  BRect real_parcel=view_parcel;
  if ((info->side==K_LEFT||info->side==K_RIGHT)&&!(info->fill&K_FILL_Y)) {
    if (info->anchor&K_BOTTOM) {
      real_parcel.top=real_parcel.bottom-info->view->Bounds().Height();
      resizingMode=resizingMode|B_FOLLOW_BOTTOM;
    } else
      resizingMode=resizingMode|B_FOLLOW_TOP;
  } else if ((info->side==K_TOP||info->side==K_BOTTOM)&&!(info->fill&K_FILL_X)) {
    if (info->anchor&K_RIGHT) {
      real_parcel.left=real_parcel.right-info->view->Bounds().Width();
      resizingMode=resizingMode|B_FOLLOW_RIGHT;
    } else 
      resizingMode=resizingMode|B_FOLLOW_LEFT;
  }
  
  // ------ fill ?
  if (info->fill&K_FILL_X) {
    info->view->ResizeTo(view_parcel.Width(),info->view->Bounds().Height());
    resizingMode=resizingMode|B_FOLLOW_LEFT_RIGHT;
  }
  if (info->fill&K_FILL_Y) {
    info->view->ResizeTo(info->view->Bounds().Width(),view_parcel.Height());
    resizingMode=resizingMode|B_FOLLOW_TOP_BOTTOM;
  }

  // ------ reshape the widget
  info->view->MoveTo(real_parcel.left,real_parcel.top);
  info->view->SetResizingMode(resizingMode);

	PRINT(("%s is %f0x%f0\n",info->view->Name(),info->view->Bounds().Width(),info->view->Bounds().Height()));
  // ------ augment view_parcel ?
  if (add_X>0) {
    ExtendBy(add_X,0);
    view_parcel.right=view_parcel.right+add_X;
    fFree.right=fFree.right+add_X;
  }
  if (add_Y>0) {
    ExtendBy(0,add_Y);
    view_parcel.bottom=view_parcel.bottom+add_Y;
    fFree.bottom=fFree.bottom+add_Y;
  }
  
  if (fFree.Height()<0) {
    ExtendBy(0,-fFree.Height()-1);
    fFree.bottom=fFree.top+3;
  }
  if (fFree.Width()<0) {
    ExtendBy(-fFree.Width()-1,0);
    fFree.right=fFree.left+3;
  }
}


void KPackView::AttachedToWindow(void)
{
	PRINT(("%s was attached\n",Name()));
  inherited::AttachedToWindow();
  // ---------
  if (Parent()!=NULL)
    SetViewColor(Parent()->ViewColor()); 
  fFree=Bounds();
  // -----------
  //Window()->DisableUpdates();
  for (int32 i=fPackQueue->CountItems();i>0;i--) {
    pack_info *args=(pack_info *)fPackQueue->ItemAt(0);
    //really_pack_child(args->view,args->side,args->anchor,args->fill,args->expand);
    really_pack_child(args);
    fPackQueue->RemoveItem((int32)0);
    //delete args;
  }
  //Window()->EnableUpdates();
  // -----------
  AttachedToWindow(!fAttachedOnce);
  if (!fAttachedOnce) {
    fAttachedOnce=true;
  }
}

void KPackView::AttachedToWindow(bool first_time)
{
}
