/*

	by Serge FANTINO - fantino@math.unice.fr
	
*/

#include <stdio.h>

#include "spliter.h"
#include "cursor.h"

#include <Window.h>
#include <Application.h>

// KSeparator <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

void draw_separator(BView *owner,orientation posture)
{
/*
	if (posture==B_HORIZONTAL) {//B_HORIZONTAL
		owner->StrokeLine(BPoint(0,2),BPoint(owner->Bounds().Width(),2));
	} else {//B_VERTICAL
		owner->StrokeLine(BPoint(2,0),BPoint(2,owner->Bounds().Height()));
	}
*/

	//FillRect(Bounds());
	rgb_color low=owner->LowColor();
	BPoint starting,ending,offset;
	if (posture==B_HORIZONTAL) {//B_HORIZONTAL
		starting=owner->Bounds().LeftTop();
		ending=owner->Bounds().RightTop();
		offset=BPoint(0,1);
	} else {//B_VERTICAL
		starting=owner->Bounds().LeftBottom();
		ending=owner->Bounds().LeftTop();
		offset=BPoint(1,0);
	}
	owner->SetLowColor(tint_color(low,B_DARKEN_3_TINT));
	owner->StrokeLine(starting,ending,B_SOLID_LOW);
	starting+=offset;
	ending+=offset;
	owner->SetLowColor(low);
	owner->StrokeLine(starting,ending,B_SOLID_LOW);
	starting+=offset;
	ending+=offset;
	owner->SetLowColor(tint_color(low,B_DARKEN_1_TINT));
	owner->StrokeLine(starting,ending,B_SOLID_LOW);
	starting+=offset;
	ending+=offset;
	owner->SetLowColor(tint_color(low,B_DARKEN_2_TINT));
	owner->StrokeLine(starting,ending,B_SOLID_LOW);
	starting+=offset;
	ending+=offset;
	owner->SetLowColor(tint_color(low,B_DARKEN_3_TINT));
	owner->StrokeLine(starting,ending,B_SOLID_LOW);
	owner->SetLowColor(low);

}

KSeparator::KSeparator(int height,orientation posture,const char *name):
BView(BRect(0,0,4,4),name,0,B_WILL_DRAW)
{
	fPosture=posture;
	SetViewColor(tint_color(LowColor(),B_DARKEN_1_TINT));
}

void KSeparator::Draw(BRect update)
{
	draw_separator(this,fPosture);
}

orientation KSeparator::Posture() {return fPosture;}

// KSplit <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

KSplit::KSplit(int height,orientation posture,const char *name):
KSeparator(height,posture,name)
{
	fTracking=false;
	fMin=fMax=1;
}

void KSplit::SetLimits(float min,float max)
{
	fBeforeHiddingSize=fMin=min;fMax=max;
}

void KSplit::AllAttached(void)
{
	//fMin=PreviousSibling()->Bounds().RightBottom();
}
 
void KSplit::Draw(BRect update)
{
	draw_separator(this,Posture());
}

void KSplit::MouseDown(BPoint where)
{
	BMessage *message=Window()->CurrentMessage();
	int32 clicks=message->FindInt32("clicks");
	if (clicks==1) {
		SetMouseEventMask(B_POINTER_EVENTS,
			B_LOCK_WINDOW_FOCUS|B_SUSPEND_VIEW_FOCUS|B_NO_POINTER_HISTORY);
		fTracking=true;
		fOffset=where;
	} else if (clicks==2) {
		BView *view=PreviousSibling();
		if (view->IsHidden()) {
			// show previous sibling
			puts("show!");
			if (Posture()==B_HORIZONTAL)
				SplitTo(Frame().LeftTop()+BPoint(0,fBeforeHiddingSize),this);
			else
				SplitTo(Frame().LeftTop()+BPoint(fBeforeHiddingSize,0),this);
		} else {
			// hide previous sibling
			SplitTo(view->Frame().LeftTop(),this);
		}
		be_app->SetCursor(B_HAND_CURSOR);
	}
}

void KSplit::MouseUp(BPoint where)
{
	fTracking=false;
	if (!Bounds().Contains(where))
		be_app->SetCursor(B_HAND_CURSOR);
}

void KSplit::MouseMoved(BPoint point,uint32 transit,const BMessage *message)
{
	if (fTracking) {// we are tracking the mouse
		BPoint left_top=ConvertToParent(point-fOffset);
		SplitTo(left_top,this);
	} else {// not tracking
			int32 buttons = 0;
       		Window()->CurrentMessage()->FindInt32("buttons", &buttons); 
       		if (!buttons) { // nobody already hold the mouse
				switch(transit) {
					case B_ENTERED_VIEW:
						if (Posture()==B_VERTICAL) be_app->SetCursor(K_VERTICAL_SPLIT_CURSOR);
						else be_app->SetCursor(K_HORIZONTAL_SPLIT_CURSOR);
						break;
					case B_EXITED_VIEW:
						be_app->SetCursor(B_HAND_CURSOR);
						break;
				}
			}
	}
}

void KSplit::SplitTo(BPoint left_top,BView *source)// left_top in parent coordinates
{
		float previous_size=0;
		float next_size=0;
		BView *previous_sibling=this,*next_sibling=this;
		// find previous/next
		previous_sibling=previous_sibling->PreviousSibling();
		next_sibling=next_sibling->NextSibling();
		bool hidde_previous_sibling=false;
		if (Posture()==B_HORIZONTAL) {// B_HORIZONTAL
			left_top.x=Frame().left;
			// compute new height for previous/next sibling
			if (previous_sibling!=NULL)
				previous_size=left_top.y-previous_sibling->Frame().top-1;
			if (next_sibling!=NULL)
				next_size=next_sibling->Frame().bottom-(left_top.y+Bounds().Height())-1;
			// resize
			// ---------------
			// if next reached it's limit, try to strech the parent view
			if (next_size<fMax&&!next_sibling->IsHidden()&&!previous_sibling->IsHidden()) {
				KSplitPane *pane=dynamic_cast<KSplitPane *>(Parent());
				previous_sibling=source->PreviousSibling();
				if (pane!=NULL&&previous_sibling!=NULL) {
					if (previous_sibling->IsHidden()) {
						previous_sibling->Show();
					}
					pane->SetFollowAllView(previous_sibling);
					pane->resize_by(0,fMax-next_size);
					pane->ResetFollowAll();
				}
			} else 
				if ((hidde_previous_sibling=(previous_size<=10&&previous_sibling!=NULL&&!previous_sibling->IsHidden()))
					||(previous_size>=fMin&&(next_size>=fMax||next_sibling->IsHidden()||previous_sibling->IsHidden()))
					||(source!=this)) {// check if there's enougth space left
				// ok, resize
				Window()->DisableUpdates();
				if (hidde_previous_sibling) {
					left_top=previous_sibling->Frame().LeftTop();
					next_size=next_sibling->Frame().bottom-(left_top.y+Frame().Height())-1;
				}
				MoveTo(left_top);
				if (next_sibling!=NULL)
					if (!next_sibling->IsHidden()) {
						next_sibling->ResizeTo(previous_sibling->Bounds().Width(),next_size);
						next_sibling->MoveTo(Frame().LeftBottom()+BPoint(0,1));
					} else {
						while ((next_sibling=next_sibling->NextSibling())!=NULL&&next_sibling->IsHidden()) {}
						KSplit *x=dynamic_cast<KSplit *>(next_sibling);
						if (x!=NULL&&x!=source) {
							x->SplitTo(Frame().LeftBottom()+BPoint(0,1),source);
						}
					}
				if (previous_sibling!=NULL) {
					if (previous_sibling->IsHidden()&&source==this) {
						// il faut resizer le previous_sibling avant...
						BView *prev_prev=previous_sibling->PreviousSibling();
						if (prev_prev!=NULL) {
							previous_sibling->MoveTo(prev_prev->Frame().LeftBottom()+BPoint(0,1));
						} else {
							previous_sibling->MoveTo(Parent()->Bounds().LeftTop());
						}
						previous_sibling->ResizeTo(Bounds().Width(),Frame().top-previous_sibling->Frame().top);
						previous_sibling->Show();
					}
					else if (hidde_previous_sibling) {
						fBeforeHiddingSize=previous_sibling->Bounds().Height();
						previous_sibling->ResizeTo(previous_sibling->Bounds().Width(),0);
						previous_sibling->Hide();
					}
					else if (!previous_sibling->IsHidden())
						previous_sibling->ResizeTo(previous_sibling->Bounds().Width(),previous_size);
				}
				Window()->EnableUpdates();
			}
		// ------------------------------------------------------------------------------------------
		} else if (Posture()==B_VERTICAL) {// B_VERTICAL
			left_top.y=Frame().top;
			// compute new height for previous/next sibling
			if (previous_sibling!=NULL)
				previous_size=left_top.x-previous_sibling->Frame().left-1;
			if (next_sibling!=NULL)
				next_size=next_sibling->Frame().right-(left_top.x+Bounds().Width())-1;
			// resize
			// ---------------
			// if next reached it's limit, try to strech the parent view
			if (next_size<fMax&&!next_sibling->IsHidden()&&!previous_sibling->IsHidden()) {
				KSplitPane *pane=dynamic_cast<KSplitPane *>(Parent());
				previous_sibling=source->PreviousSibling();
				if (pane!=NULL&&previous_sibling!=NULL) {
					if (previous_sibling->IsHidden()) {
						previous_sibling->Show();
					}
					pane->SetFollowAllView(previous_sibling);
					pane->resize_by(fMax-next_size,0);
					pane->ResetFollowAll();
				}
			} else 
				if ((hidde_previous_sibling=(previous_size<=10&&previous_sibling!=NULL&&!previous_sibling->IsHidden()))
					||(previous_size>=fMin&&(next_size>=fMax||next_sibling->IsHidden()||previous_sibling->IsHidden()))
					||(source!=this)) {// check if there's enougth space left
				// ok, resize
				Window()->DisableUpdates();
				if (hidde_previous_sibling) {
					left_top=previous_sibling->Frame().LeftTop();
					next_size=next_sibling->Frame().right-(left_top.x+Frame().Width())-1;
				}
				MoveTo(left_top);
				if (next_sibling!=NULL)
					if (!next_sibling->IsHidden()) {
						next_sibling->ResizeTo(next_size,previous_sibling->Bounds().Height());
						next_sibling->MoveTo(Frame().RightTop()+BPoint(1,0));
					} else {
						while ((next_sibling=next_sibling->NextSibling())!=NULL&&next_sibling->IsHidden()) {}
						KSplit *x=dynamic_cast<KSplit *>(next_sibling);
						if (x!=NULL&&x!=source) {
							x->SplitTo(Frame().RightTop()+BPoint(1,0),source);//0,1
						}
					}
				if (previous_sibling!=NULL) {
					if (previous_sibling->IsHidden()&&source==this) {
						// il faut resizer le previous_sibling avant...
						BView *prev_prev=previous_sibling->PreviousSibling();
						if (prev_prev!=NULL) {
							previous_sibling->MoveTo(prev_prev->Frame().RightTop()+BPoint(1,0));
						} else {
							previous_sibling->MoveTo(Parent()->Bounds().LeftTop());
						}
						previous_sibling->ResizeTo(Frame().left-previous_sibling->Frame().left,Bounds().Height());
						previous_sibling->Show();
					}
					else if (hidde_previous_sibling) {
						// hide previous
						fBeforeHiddingSize=previous_sibling->Bounds().Width();
						previous_sibling->ResizeTo(0,previous_sibling->Bounds().Height());
						previous_sibling->Hide();
					}
					else if (!previous_sibling->IsHidden())
						previous_sibling->ResizeTo(previous_size,previous_sibling->Bounds().Height());
				}
				Window()->EnableUpdates();
			} else if (previous_sibling->IsHidden()&&previous_size<0) {
				puts("last");
				// try to pass control to previous splitter
				KSplit *previous_split=cast_as(previous_sibling->PreviousSibling(),KSplit);
				if (previous_split!=NULL) {
					previous_split->SplitTo(left_top,previous_split);
				}
			}
		}// endif VERTICAL
}

// KSplitPane <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

struct simple_pack_info {

	simple_pack_info(BView *w)
	{
		view=w;bounds=w->Bounds();
	};

	BView *view;
	BRect bounds;
}; 

KSplitPane::KSplitPane(BRect rect,const char *name,orientation posture,uint32 resizingMode,uint32 flags)
:BView(rect,name,resizingMode,flags|B_FRAME_EVENTS)
{
	fPosture=posture;
	fQueue=new BList();
	fFirstTime=true;
}

KSplitPane::~KSplitPane()
{
	delete fQueue;
}
	
orientation KSplitPane::Posture() {return fPosture;}
	
void KSplitPane::SetPosture(orientation posture)
{
	if (posture!=fPosture) {
		fPosture=posture;
		if (Window()!=NULL) {// update on screen
			// remove everything
			BView *child;
			for (int32 count=CountChildren();count>0;--count) {
				child=ChildAt(0);
				child->RemoveSelf();
				if (dynamic_cast<KSplit *>(child)!=NULL) delete child;
			};
			// repack everything
			pack_everything();
		}
	}
}

void KSplitPane::AttachedToWindow(void)
{
	if (fFirstTime) {
		fFirstTime=false;
		if (Parent()!=NULL) {
			SetViewColor(Parent()->ViewColor());
			SetLowColor(Parent()->LowColor());
			SetHighColor(Parent()->HighColor());
		};
		if (Window()!=NULL) {
			// repack everything
			pack_everything();
		}
	}
}

void KSplitPane::AddChild(BView *view)
{
	simple_pack_info *info=new simple_pack_info(view);
	simple_pack_info *previous=(simple_pack_info *)(fQueue->LastItem());
	fQueue->AddItem(info);	
	if (Window()!=NULL)
		really_add_child(info,previous);
}

status_t KSplitPane::HideChild(BView *view)
{
	int32 index=IndexOf(view);
	if (index<0||view->IsHidden()) return B_ERROR;
	KSplit *split=cast_as(view->NextSibling(),KSplit);
	if (split!=NULL) {
		split->SplitTo(view->Frame().LeftTop(),split);
		return B_OK;
	}
	return B_ERROR;
}

status_t KSplitPane::RemoveChild(BView *view)
{
	int32 index=IndexOf(view);
	if (index<0) return B_ERROR;
	// remove everything
	BView *child;
	for (int32 count=CountChildren();count>0;--count) {
		child=ChildAt(0);
		child->RemoveSelf();
		if (dynamic_cast<KSplit *>(child)!=NULL) delete child;
	};
	// remove the view
	fQueue->RemoveItem(index);
	// repack everything
	pack_everything();
	return B_OK;
}

int32 KSplitPane::IndexOf(BView *view)
{
	int32 result=-1;
	for (int32 index=0,count=fQueue->CountItems();count>0;--count,++index) {
		if (((simple_pack_info *)(fQueue->ItemAt(index)))->view==view) {
			puts("found child !");
			result=index;
			break;
		}
	}
	return result;
}

void KSplitPane::MessageReceived(BMessage *message)
{
	switch (message->what) {
		case K_SET_HORIZONTAL:{
			SetPosture(B_HORIZONTAL);
			void *pointer;
			if (message->FindPointer("source",&pointer)==B_OK) {
				BMenuItem *item=cast_as((BMenuItem *)pointer,BMenuItem);
				item->SetMarked(true);
			}
			break;
		}
		case K_SET_VERTICAL:{
			SetPosture(B_VERTICAL);
			void *pointer;
			if (message->FindPointer("source",&pointer)==B_OK) {
				BMenuItem *item=cast_as((BMenuItem *)pointer,BMenuItem);
				item->SetMarked(true);
			}
			break;
		}
		default:
			inherited::MessageReceived(message);
	}
}

void KSplitPane::SetFollowAllView(BView *view)
{
	uint32 resizing_mode;
	view->SetResizingMode(B_FOLLOW_ALL);
	// ---
	if (fPosture==B_HORIZONTAL)
		resizing_mode=B_FOLLOW_LEFT_RIGHT|B_FOLLOW_TOP;
	else if (fPosture==B_VERTICAL)
		resizing_mode=B_FOLLOW_LEFT|B_FOLLOW_TOP_BOTTOM;
	BView *previous_sibling=view;
	while ((previous_sibling=previous_sibling->PreviousSibling())!=NULL)
		previous_sibling->SetResizingMode(resizing_mode);
	// ---
	if (fPosture==B_HORIZONTAL)
		resizing_mode=B_FOLLOW_LEFT_RIGHT|B_FOLLOW_BOTTOM;
	else if (fPosture==B_VERTICAL)
		resizing_mode=B_FOLLOW_RIGHT|B_FOLLOW_TOP_BOTTOM;
	BView *next_sibling=view;
	while ((next_sibling=next_sibling->NextSibling())!=NULL)
		next_sibling->SetResizingMode(resizing_mode);
}

void KSplitPane::ResetFollowAll()
{
	uint32 resizing_mode;
	if (fPosture==B_HORIZONTAL)
		resizing_mode=B_FOLLOW_LEFT_RIGHT|B_FOLLOW_TOP;
	else if (fPosture==B_VERTICAL)
		resizing_mode=B_FOLLOW_LEFT|B_FOLLOW_TOP_BOTTOM;
	// --
	BView *view=NULL;
	for (int32 index=0,count=CountChildren();index<count;++index)
		(view=ChildAt(index))->SetResizingMode(resizing_mode);
	// --
	if (view!=NULL) view->SetResizingMode(B_FOLLOW_ALL);
}
	
void KSplitPane::really_add_child(simple_pack_info *info,simple_pack_info *previous)
{
	BView *view=info->view;
	if (view->IsHidden()) view->Show();
	if (previous!=NULL) {// NOT THE FIRST VIEW
		BView *last=previous->view;
		puts("not null");
		// ------ restore previous last view
		if (fPosture==B_HORIZONTAL) {
			last->ResizeBy(0,previous->bounds.Height()-last->Bounds().Height());
			last->SetResizingMode(B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP);
		} else {
			last->ResizeBy(previous->bounds.Width()-last->Bounds().Width(),0);
			last->SetResizingMode(B_FOLLOW_LEFT | B_FOLLOW_TOP_BOTTOM);
		}
		// ------ build split
		KSplit *s=new KSplit(4,fPosture,"__");
		inherited::AddChild(s);
		if (fPosture==B_HORIZONTAL) {
			s->MoveTo(last->Frame().LeftBottom()+BPoint(0,1));
			s->ResizeTo(Bounds().right,4);
			s->SetResizingMode(B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP);
			s->SetLimits(previous->bounds.Height(),info->bounds.Height());
		} else {
			s->MoveTo(last->Frame().RightTop()+BPoint(1,0));
			s->ResizeTo(4,Bounds().bottom);
			s->SetResizingMode(B_FOLLOW_LEFT | B_FOLLOW_TOP_BOTTOM);
			s->SetLimits(previous->bounds.Width(),info->bounds.Width());
		}
		// ------ add view
		view->SetResizingMode(B_FOLLOW_ALL);
		inherited::AddChild(view);
		if (fPosture==B_HORIZONTAL) {
			view->MoveTo(s->Frame().LeftBottom()+BPoint(0,1));
		} else {
			view->MoveTo(s->Frame().RightTop()+BPoint(1,0));
		}
		float height=Bounds().bottom-view->Frame().top;
		float width=Bounds().right-view->Frame().left;
		if (fPosture==B_HORIZONTAL) {
			view->ResizeTo(Bounds().RightBottom().x,height);
		} else {
			view->ResizeTo(width,Bounds().RightBottom().y);
		}
		if (height<info->bounds.Height()) {
			resize_by(0,info->bounds.Height()-height);
		}
		if (width<info->bounds.Width()) {
			resize_by(info->bounds.Width()-width,0);
		}
	} else {// IT's THE FIRST VIEW
		puts("null");
		inherited::AddChild(view);
		view->MoveTo(0,0);
		view->ResizeTo(Bounds().RightBottom().x,Bounds().RightBottom().y);
		view->SetResizingMode(B_FOLLOW_ALL);
	}
}

void KSplitPane::pack_everything()
{
	// repack everything
	simple_pack_info *previous=NULL,*current=NULL;
	for (int32 index=0,count=fQueue->CountItems();count>0;++index,--count) {
		current=(simple_pack_info *)fQueue->ItemAt(index);
		really_add_child(current,previous);
		previous=current;
	}
} 

void KSplitPane::resize_by(float x,float y)
{
//	ResizeBy(x,y);
///*
	BView *target=this;
	while (target!=NULL&&(x!=0||y!=0)) {
		uint32 resizing_mode=target->ResizingMode();
		if (x!=0) {
			if (resizing_mode&B_FOLLOW_LEFT_RIGHT||resizing_mode&B_FOLLOW_ALL) {
			
			} else {
				ResizeBy(x,0);
				x=0;
			}
		}
		if (y!=0) {
			if (resizing_mode&B_FOLLOW_TOP_BOTTOM||resizing_mode&B_FOLLOW_ALL) {
			
			} else {
				ResizeBy(y,0);
				y=0;
			}
		}
		target=target->Parent();
	}
	// ---
	if (target==NULL&&(x!=0||y!=0)) {
		BScreen screen;
		BPoint right_bottom=Window()->Frame().RightBottom()+BPoint(x,y);
		if (screen.Frame().Contains(right_bottom))
			Window()->ResizeBy(x,y);
	}
//*/
}



