//--------------------------------------------------------------------------
//
//			by Serge Fantino
//			fantino@math.unice.fr
//
//		Copyright 1998-1999, All Rights Reserved.
//
//--------------------------------------------------------------------------

#include "tips.h"

KTipsWindow::KTipsWindow(BView *owner,BPoint where,const char *text):
BWindow(BRect(where.x-20,where.y+10,where.x+50,where.y+30),"",B_BORDERED_WINDOW_LOOK,B_FLOATING_SUBSET_WINDOW_FEEL,B_AVOID_FRONT)
{
	BRect bounds=Bounds();
	bounds.InsetBy(3,3);
	BTextView *bg=new BTextView(Bounds(),"",bounds,B_FOLLOW_ALL,B_WILL_DRAW);
	AddChild(bg);
	BFont font;
	bg->GetFont(&font);
	font_height height;
	font.GetHeight(&height);
	ResizeBy(font.StringWidth(text)-50,height.ascent+height.descent-15);
	bg->SetViewColor(255,255,150);
	//bg->SetViewColor(B_TRANSPARENT_32_BIT);
	bg->MakeResizable(true);
	bg->MakeEditable(false);
	bg->MakeSelectable(false);
	bg->SetText(text);
	// ---
	AddToSubset(owner->Window());
	Show();
	// ---
	/*
	Lock();
	bg->SetHighColor(255,255,150);
	bg->SetDrawingMode(B_OP_BLEND);
	bg->FillRect(Bounds());
	bg->SetLowColor(255,255,150);
	bg->SetDrawingMode(B_OP_COPY);
	Unlock();
*/
} 
	
void KTipsWindow::Quit()
{
	snooze(10000);
	BWindow::Quit();
}

// KTips <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

KTips::KTips(int32 ID,const char *tipsText,BView *owner,KTipsManager *manager)
:BMessageFilter(B_MOUSE_MOVED)
{
	fID=ID;
	fTipsText.SetTo(tipsText);
	fOwner=owner;
	fManager=manager;
	// ---
	//owner->AddFilter(this);
}

const char *KTips::Text()
{
	return fTipsText.String();
}
	
filter_result KTips::Filter(BMessage *message, BHandler **target)
{ 
	//message->PrintToStream();
	int32 transit=message->FindInt32("be:transit");
	BPoint where=message->FindPoint("be:view_where");
	int32 buttons=message->FindInt32("buttons");
	switch (transit) {
		case B_ENTERED_VIEW:{
			PRINT(("entered %s\n",(*target)->Name()));
			where=fOwner->ConvertToScreen(where);
			float bottom=fOwner->ConvertToScreen(fOwner->Bounds()).bottom;
			//if (bottom-where.y<50)
				where.y=bottom;
			if (buttons==0) fManager->ShowTips(this,where);
			break;}
		case B_EXITED_VIEW:
			PRINT(("exited %s\n",(*target)->Name()));
			if (true||!fOwner->Bounds().Contains(where)||fOwner->CountChildren()==0)
				fManager->HideTips();
			break;
	}
	return B_DISPATCH_MESSAGE;
}

// <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

class KEmbeddedMessageFilter : public BMessageFilter {
public:

	KEmbeddedMessageFilter(KTipsManager *tm,uint32 what,filter_hook filter)
	:BMessageFilter(what,filter)
	{
		fTipsManager=tm;
	}
	
	KTipsManager	*fTipsManager;
};

KTipsManager::KTipsManager(BWindow *win,bool enabled)
{
	fWindow=win;
	fEnabled=enabled;
	fTipsWindow=NULL;
	fLastID=fCurrentID=fID=0;
	fCurrentTips=NULL;
	fNeedToShow=false;
	//
	win->AddCommonFilter(new KEmbeddedMessageFilter(this,B_PULSE,filter));
	//win->AddFilter(new KEmbeddedMessageFilter(this,B_MOUSE_MOVED,filter));
}

void KTipsManager::AddTips(const char *tips,BView *owner)
{
	++fID;
	create_tips(fID,tips,owner,owner);
	// ---
	BView *child; 
  	if ( child = owner->ChildAt(0) ) { 
		while ( child ) {
			create_tips(fID,tips,owner,child);
			child = child->NextSibling(); 
       	} 
   	}
}

void KTipsManager::create_tips(int32 ID,const char *tips,BView *owner,BView *target)
{
	target->AddFilter(new KTips(ID,tips,owner,this));
	target->AddFilter(new KEmbeddedMessageFilter(this,B_MOUSE_MOVED,filter));
	target->AddFilter(new KEmbeddedMessageFilter(this,B_MOUSE_DOWN,filter));
}
	
void KTipsManager::ShowTips(KTips *tips,BPoint where)
{
	if (!fEnabled) return;
	if (tips->fID!=fCurrentID) {
		if (fCurrentID!=0) {
			HideTips();
		}
		fCurrentTips=tips;
		fWhere=where;
		fNeedToShow=true;
		fWait=false;
		//fTipsWindow=new KTipsWindow(where,tips->Text());
		//fTipsWindow->AddToSubset(tips->fOwner->Window());
		//fActive=true;
		//fCurrentID=tips->fID;
	}
	//puts(fTipsText.String());
}
	
void KTipsManager::HideTips()
{
	PRINT(("hide"));
	if (fTipsWindow!=NULL) {
		fTipsWindow->Lock();
		fTipsWindow->Quit();
	}
	fTipsWindow=NULL;
	fActive=false;
	fCurrentID=0;
	fCurrentTips=NULL;
}
	
void KTipsManager::Pulse()
{
	if (fNeedToShow&&fCurrentTips!=NULL&&!fWait&&fWindow->IsFront()&&fLastID!=fCurrentTips->fID) {
		BPoint where;
		uint32 buttons;
		fCurrentTips->fOwner->GetMouse(&where,&buttons,false);
		if (buttons==0) {
			where=fCurrentTips->fOwner->ConvertToScreen(where);
			fTipsWindow=new KTipsWindow(fCurrentTips->fOwner,where,fCurrentTips->Text());
			fTipsWindow->AddToSubset(fCurrentTips->fOwner->Window());
			fActive=true;
			fLastID=fCurrentID=fCurrentTips->fID;
			fNeedToShow=false;
		}
	} else if (fCurrentTips!=NULL&&fLastID==fCurrentTips->fID) {
		fLastID=0;
	}
	fWait=false;
}

filter_result KTipsManager::filter(BMessage *message, 
         BHandler **target, 
         BMessageFilter *messageFilter)
{
	KTipsManager *myself=((KEmbeddedMessageFilter *)messageFilter)->fTipsManager;
	switch (message->what) {
		case B_MOUSE_MOVED:
			if (!message->HasInt32("modifiers")) {
				// it's a real mouse move
				if (myself->fActive)
					myself->HideTips();
				else
					myself->fWait=true;
			} else {
			}
			break;
		case B_PULSE:
			myself->Pulse();
			break;
		case B_MOUSE_DOWN:
			if (myself->fActive) myself->HideTips();
			break;
	}
	return B_DISPATCH_MESSAGE;
}
