//--------------------------------------------------------------------------
//
//			by Serge Fantino
//			fantino@mad.unice.fr
//
//		Copyright 1998, All Rights Reserved.
//
//--------------------------------------------------------------------------

#include <stdio.h>
#include <alloc.h>

#include "wDBObjects.h"

#include "constant.h"

#include "wMessageUtils.h"

class wDBLooper : public BLooper {
public:
	wDBLooper():BLooper("DBLooper")
	{
		Run();
	}
};

extern wDBLooper *DBLooper=new wDBLooper();

wTable::wTable(const char *name):BHandler(name)
{
	fRecord=NULL;
	fDataRecord=new BMessage();
	fDBWidgets=new BList();
	// ---
	if (DBLooper->Lock()) {
		DBLooper->AddHandler(this);
		DBLooper->Unlock();
	}
}

wTable::~wTable()
{
}

void wTable::CloseTable()
{
	for (int32 i=fDBWidgets->CountItems()-1;i>=0;i--) {
		wDBWidget *w=(wDBWidget *)fDBWidgets->ItemAt(i);
		w->ClosingTable();
	}
	delete this;
}

wRecord *wTable::CurrentRecord() {return fRecord;}
BMessage *wTable::CurrentDataRecord() {return fDataRecord;}

void wTable::SetCurrentRecord(wRecord *record)
{
	bool selectionChanged=(fRecord!=record);
	fRecord=record;
	fDataRecord->MakeEmpty();
	if (fRecord!=NULL) fRecord->GetData(fDataRecord);
	for (int32 i=fDBWidgets->CountItems()-1;i>=0;i--) {
		wDBWidget *w=(wDBWidget *)fDBWidgets->ItemAt(i);
		w->UpdateCurrentRecord(selectionChanged);
	}
}

void wTable::RemoveRecord(wRecord *record)
{
	if (record!=NULL) {
		for (int32 i=fDBWidgets->CountItems()-1;i>=0;i--) {
			wDBWidget *w=(wDBWidget *)fDBWidgets->ItemAt(i);
			w->RemoveRecord(record);
		}
	if (record==fRecord) SetCurrentRecord(NULL);
	}
}
	
	
void wTable::AddDBWidget(wDBWidget *w)
{
	fDBWidgets->AddItem(w);
}

void wTable::RemoveDBWidget(wDBWidget *w)
{
	fDBWidgets->RemoveItem(w);
}

void wTable::MessageReceived(BMessage *message)
{
	switch (message->what) {
		case K_UPDATE:{
			const char *field_name;
			if (message->FindString("field_name",&field_name)!=B_OK)
				field_name="";
			int32 field_index=message->FindInt32("field_index");
			if (fRecord!=NULL)
				fRecord->SetData(fDataRecord,field_name,field_index);
			BMessage update(K_UPDATE);
			for (int32 i=fDBWidgets->CountItems()-1;i>=0;i--) {
				//wDBWidget *w=(wDBWidget *)fDBWidgets->ItemAt(i);
				//w->Looper()->PostMessage(&update,w);
				wDBWidget *w=(wDBWidget *)fDBWidgets->ItemAt(i);
				BHandler *h=dynamic_cast<BHandler *>(w);
				if (h!=NULL)
					h->Looper()->PostMessage(&update,h);
			}
			break;
		}
		default:
			BHandler::MessageReceived(message);
	}
}


// ----------------------------------------------------------------

wDBWidget::wDBWidget(wTable *table)
{
	fTable=table;
	if (table!=NULL) {
	table->AddDBWidget(this);
	}
	fDataRecord=NULL;
	fRecord=NULL;
}

wDBWidget::~wDBWidget()
{
	LockRecord(false);
	if (fTable!=NULL) {
		fTable->RemoveDBWidget(this);
	}
}

status_t wDBWidget::SendToTable(BMessage *msg)
{
	if (fTable!=NULL) {
		status_t err=fTable->Looper()->PostMessage(msg,fTable);
		return err;
	} else return B_ERROR;
}

wTable *wDBWidget::Table() {return fTable;}
BMessage *wDBWidget::Data() {return fDataRecord;}

void wDBWidget::LockRecord(bool lock) 
{
	fLockRecord=lock;
	if (fRecord!=NULL) {
		fRecord->Lock(lock);
	}
}
bool wDBWidget::IsRecordLocked() {return fLockRecord;}

void wDBWidget::SelectRecord(wRecord *record)
{
	if (fTable!=NULL)
		fTable->SetCurrentRecord(record);
}

void wDBWidget::SyncRecord()
{
	SelectRecord(fRecord);
}

void wDBWidget::SendUpdate(const char *fieldName,int32 fieldIndex)
{
	BMessage update(K_UPDATE);
	update.AddString("field_name",fieldName);
	update.AddInt32("field_index",fieldIndex);
	SendToTable(&update);
}

void wDBWidget::UpdateCurrentRecord(bool selectionChanged)
{
	if (selectionChanged&&!fLockRecord) {
		if (Table()!=NULL) {
			fRecord=Table()->CurrentRecord();
			fDataRecord=Table()->CurrentDataRecord();
		} else {
			fRecord=NULL;
			fDataRecord=NULL;
		}
	}
}

void wDBWidget::RemoveRecord(wRecord *record)
{
	if (fLockRecord&&record==fRecord) {
		fLockRecord=false;	
	}
}

void wDBWidget::ClosingTable(){}

// ---------------

wDBFieldWidget::wDBFieldWidget(wTable *table,const char *fieldName,int32 fieldIndex):
wDBWidget(table)
{
	fFieldName=(char *)calloc(strlen(fieldName)+1,sizeof(char));
	strcpy(fFieldName,fieldName);
	fFieldIndex=fieldIndex;
}

wDBFieldWidget::~wDBFieldWidget()
{
	free(fFieldName);
}

const char *wDBFieldWidget::FieldName() {return fFieldName;}
int32 wDBFieldWidget::FieldIndex() {return fFieldIndex;}
int32 wDBFieldWidget::FieldType() {return fFieldType;}

void wDBFieldWidget::UpdateCurrentRecord(bool selectionChanged)
{
	wDBWidget::UpdateCurrentRecord(selectionChanged);
	if (selectionChanged&&(Data()==NULL||Data()->GetInfo(fFieldName,&fFieldType)!=B_OK))
		fFieldType=0;
}

void wDBFieldWidget::SendUpdate()
{
	wDBWidget::SendUpdate(fFieldName,fFieldIndex);
}

status_t wDBFieldWidget::SetData(void *data,int32 size)
{
	if (Data()==NULL) return B_ERROR;
	return Data()->ReplaceData(FieldName(),FieldType(),FieldIndex(),data,size);
}
	
// ----------------------------------------------------------------

wFieldControl::wFieldControl(BRect frame, const char *name, 
  	const char *label,wTable *table,const char *fieldName,int32 fieldIndex,
   	BMessage *message,uint32 resizingMode,uint32 flags):
inherited(frame,name,label,"",message,resizingMode,flags),
wDBFieldWidget(table,fieldName,fieldIndex)
{
	SetEnabled(false);
}

void wFieldControl::MessageReceived(BMessage *message)
{
	switch (message->what) {
		case K_UPDATE:
			UpdateCurrentRecord(false);
			break;
		case B_SIMPLE_DATA:
			if (message->WasDropped()&&Data()!=NULL) {
				char *name;
				uint32 type;
				if (message->GetInfo(FieldType(),0,&name,&type)==B_OK) {
					void *data;
					int32 size;
					if ((message->FindData(name,type,0,&data,&size)==B_OK)&&
						SetData(data,size)) {
						//UpdateCurrentRecord(false);
						SendUpdate();
  					}
  					break;
				}
			}
		default:
			inherited::MessageReceived(message);
	}
}
  	
status_t wFieldControl::Invoke(BMessage *message) 
{
	BPoint point;
	BRect rect;
	if (Data()!=NULL) {
		switch (FieldType()) {
  			case B_STRING_TYPE:
				Data()->ReplaceString(FieldName(),FieldIndex(),Text());
				break;
			case B_INT32_TYPE:
			case B_INT16_TYPE:
			case B_INT8_TYPE:
				int32 i;
				sscanf(Text(),"%d",&i);
				Data()->ReplaceInt32(FieldName(),FieldIndex(),i);
				break;
			case B_POINT_TYPE:
				sscanf(Text(),"%f,%f",&point.x,&point.y);
				Data()->ReplacePoint(FieldName(),FieldIndex(),point);
				break;
			case B_RECT_TYPE:
				//float x,y;
				sscanf(Text(),"%f,%f,%f,%f",&rect.left,&rect.top,&rect.right,&rect.bottom);
				Data()->ReplaceRect(FieldName(),FieldIndex(),rect);
				break;
		}
		// ---
		SendUpdate();
  	}
 	return inherited::Invoke();
}
  	
void wFieldControl::UpdateCurrentRecord(bool selectionChanged)
{
	wDBFieldWidget::UpdateCurrentRecord(selectionChanged);
	char *value="";
  	char buf[200];
  	SetEnabled(false);
  	switch (FieldType()) {
  		case 0:
  			SetText("");
			SetEnabled(false);
			break;
 	 	case B_STRING_TYPE:
  			SetEnabled(Data()->FindString(FieldName(),FieldIndex(),&value)==B_OK);
  			break;
  		case B_INT32_TYPE:
		case B_INT16_TYPE:
		case B_INT8_TYPE:
  		case B_RECT_TYPE:
  		case B_POINT_TYPE:
  			SetEnabled(true);
  		default:
  			sprint_data(buf,FieldType(),Data(),FieldName(),FieldIndex());
  			value=buf;
  	}
  	SetText(value);
}
