//--------------------------------------------------------------------------
//
//			by Serge Fantino
//			fantino@mad.unice.fr
//
//		Copyright 1998, All Rights Reserved.
//
//--------------------------------------------------------------------------

#ifndef DBOBJECT_KMSF
#define DBOBJECT_KMSF

#include <AppKit.h>
#include <InterfaceKit.h>

class wDBWidget;

class wRecord {
public:
	wRecord(){};
	
	virtual void GetData(BMessage *data)=0;
	virtual void SetData(BMessage *data,const char *fieldName,int32 fieldIndex)=0;
	
	virtual const char *RecordName(){return "";}
	
	void Lock(bool lock) {fLock=lock;}
	bool IsLocked() {return fLock;}
private:
	bool	fLock;
};

class wTable : public BHandler {
public:
	wTable(const char *name);
	~wTable();
	void CloseTable();
	
	wRecord *CurrentRecord();
	void SetCurrentRecord(wRecord *record);
	void RemoveRecord(wRecord *record);
	
	BMessage *CurrentDataRecord();
	
	void AddDBWidget(wDBWidget *);
	void RemoveDBWidget(wDBWidget *);
	
	virtual void MessageReceived(BMessage *msg);
	
private:
    BMessage	*fDataRecord;   
    wRecord		*fRecord;
    BList		*fDBWidgets;
};

class wDBWidget {
public:
	wDBWidget(wTable *table);
	~wDBWidget();
	
	status_t SendToTable(BMessage *msg);
	wTable *Table();
	BMessage *Data();
	
	void LockRecord(bool lock);
	bool IsRecordLocked();
	
	void SelectRecord(wRecord *record);
	void SyncRecord();
	
	//virtual void SetRecord(BMessage *record){};
	virtual void UpdateCurrentRecord(bool selectionChanged);
	virtual void RemoveRecord(wRecord *record);
	void SendUpdate(const char *fieldName,int32 fieldIndex);
	
	virtual void ClosingTable();
	
private:
	wTable		*fTable;
	wRecord		*fRecord;
	BMessage	*fDataRecord;
	bool		fLockRecord;
};

class wDBFieldWidget : public wDBWidget {
public:
	wDBFieldWidget(wTable *table,const char *fieldName,int32 fieldIndex);
	~wDBFieldWidget();
	
	const char *FieldName();
	int32 FieldIndex();
	int32 FieldType();
	
	//virtual void SetRecord(BMessage *record){};
	virtual status_t SetData(void *data,int32 size);
	virtual void UpdateCurrentRecord(bool selectionChanged);
	virtual void SendUpdate();

private:
	char		*fFieldName;
	int32		fFieldIndex;
	uint32		fFieldType;
};

class wFieldControl : public wDBFieldWidget, public BTextControl { 
public:
	wFieldControl(BRect frame, const char *name,const char *label,
		wTable *table,const char *fieldName,int32 fieldIndex,
        BMessage *message, 
        uint32 resizingMode = B_FOLLOW_LEFT | B_FOLLOW_TOP, 
        uint32 flags = B_WILL_DRAW | B_NAVIGABLE);
    
  	virtual void MessageReceived(BMessage *message);
  
  	virtual status_t Invoke(BMessage *message = NULL);
  	
  	virtual void UpdateCurrentRecord(bool selectionChanged);
  	
private:
	typedef BTextControl inherited;
	char		*fFieldName;
	int32		fFieldIndex;
	uint32		fFieldType;
	BMessage	*fRecord;
};

#endif
