//--------------------------------------------------------------------------
//
//			by Serge Fantino
//			fantino@mad.unice.fr
//
//		Copyright 1998, All Rights Reserved.
//
//--------------------------------------------------------------------------

#include <stdio.h>
#include <alloc.h>

#include "wDataEditor.h"

#include "constant.h"

// - LM
#include "tab_view.h"
#include "scroll_view.h"

#include "wMessageUtils.h"
#include "wDBObjects.h"

#include "wSpecialisedEditor.h"
#include "wMessageBuilder.h"

wPropertyItem::wPropertyItem(BMessage *msg,const char *name,type_code type):BStringItem(""),wRecord()
{
	fName=NULL;
	fOriginalName=NULL;
	fMessage=msg;
	fType=type;
	fCountItems=0;
	SetName(name);
	// ---
	UpdateText();
}

wPropertyItem::~wPropertyItem()
{
	free(fName);
	free(fOriginalName);
}

void wPropertyItem::UpdateText()
{
	char buf[200];
   	sprintf(buf,"%s [%s]",fName,name_of_data_type(fType));
	SetText(buf);
}
	
const char *wPropertyItem::OriginalName() {return fOriginalName;}
const char *wPropertyItem::Name() {return fName;}
void wPropertyItem::SetName(const char *name)
{
	free(fName);
	fName=(char *)calloc(strlen(name)+1,sizeof(char));
	strcpy(fName,name);
	if (fCountItems==0) {// if no items, set original name too
		free(fOriginalName);
		fOriginalName=(char *)calloc(strlen(name)+1,sizeof(char));
		strcpy(fOriginalName,name);
	}
	UpdateText();
}

type_code wPropertyItem::Type() {return fType;}
BMessage *wPropertyItem::Message() {return fMessage;}
int32 wPropertyItem::CountItems() {return fCountItems;}
	
status_t wPropertyItem::ExtractMessage(BMessage *result,int32 index)
{
	switch (fType) {
		case B_MESSAGE_TYPE:
			return (fMessage->FindMessage(fOriginalName,index,result));
		default:
			return B_ERROR;
	}
}
	
status_t wPropertyItem::ExtractData(void **data,int32 index,ssize_t *size)
{
	return fMessage->FindData(fOriginalName,fType,index,data,size);
}
	
status_t wPropertyItem::MimicMessage(BMessage *mimic,int32 index)
{
	void *data;
	ssize_t size;
	status_t err;
	if ((err=ExtractData(&data,index,&size))==B_OK) {
		return mimic->AddData(fName,fType,data,size,false,fCountItems);
	} else {
		return mimic->AddData(fName,fType,data,0,false,fCountItems);
	}
	//return err;
}
	
int32 wPropertyItem::AddItem(void *item) {return ++fCountItems;}

void wPropertyItem::GetData(BMessage *message)
{
	message->AddString("name",fName);
	message->AddInt32("type",fType);
	message->AddInt32("countItems",fCountItems);
}

void wPropertyItem::SetData(BMessage *message,const char *fieldName,int32 fieldIndex)
{
	if (strcmp(fieldName,"name")==0)
		SetName(message->FindString(fieldName,fieldIndex));
	else if (strcmp(fieldName,"type")==0) {
		fType=message->FindInt32(fieldName,fieldIndex);
		UpdateText();
	}
}

const char *wPropertyItem::RecordName()
{
	return Name();
}
	
// --------------------------------------------------------------------------------------

wDataItem::wDataItem(wPropertyItem *propertyItem,int32 index):BStringItem(""),wRecord()
{
	fPropertyItem=propertyItem;
	fIndex=index;
	fPropertyItem->AddItem(this);
	UpdateText();
}

void wDataItem::UpdateText() 
{
	if (fPropertyItem->Message()->HasData(fPropertyItem->Name(),fPropertyItem->Type(),fIndex)) {
		char buf[200];
		sprint_data(buf,fPropertyItem->Type(),fPropertyItem->Message(),fPropertyItem->Name(),fIndex);
		SetText(buf);
	} else {
		SetText("- entry not defined -");
	}
}
	
int32 wDataItem::Index() {return fIndex;}
	
wPropertyItem *wDataItem::PropertyItem() {return fPropertyItem;}
	
status_t wDataItem::ExtractMessage(BMessage *result)
{
	return fPropertyItem->ExtractMessage(result,fIndex);
}
	
status_t wDataItem::ExtractData(void **data,ssize_t *size)
{
	return fPropertyItem->ExtractData(data,fIndex,size);
}
	
bool wDataItem::InitiateDrag(BView *view,BRect frame)
{
	BMessage extract;
	if (ExtractMessage(&extract)==B_OK) {
		view->DragMessage(&extract,frame,view);
		return true;
	} else
		return false;
}
	
status_t wDataItem::MimicMessage(BMessage *mimic)
{
	return fPropertyItem->MimicMessage(mimic,fIndex);
}

void wDataItem::GetData(BMessage *message)
{
	fPropertyItem->GetData(message);
	message->AddBool("hasData",true);
	void *data;
	ssize_t size;
	status_t err;
	if ((err=fPropertyItem->ExtractData(&data,fIndex,&size))==B_OK) {
		message->AddData("data",fPropertyItem->Type(),data,size,false,1);
	} else {
		message->AddData("data",fPropertyItem->Type(),"",1,false,1);
	}
}

void wDataItem::SetData(BMessage *message,const char *fieldName,int32 fieldIndex)
{
	if (strcmp(fieldName,"data")==0) {
		// extract from message and replace...
		void *data;
		ssize_t size;
		status_t err;
		if (message->FindData("data",fPropertyItem->Type(),0,&data,&size)==B_OK) {
			err=fPropertyItem->Message()->ReplaceData(fPropertyItem->OriginalName(),fPropertyItem->Type(),fIndex,data,size);
			if (err!=B_OK) {
				fPropertyItem->Message()->AddData(fPropertyItem->OriginalName(),fPropertyItem->Type(),data,size);
			}
		}
		UpdateText();
	} else
		fPropertyItem->SetData(message,fieldName,fieldIndex);
}

const char *wDataItem::RecordName()
{
	return fPropertyItem->Name();
}

// ----------------------------------------------------------------

wDataEditor::wDataEditor(BRect frame, const char *name, 
		wTable *table,
		//bool propagate,
		uint32 resizingMode,uint32 flags):
wDBWidget(table)
,inherited(frame,name,resizingMode,flags,B_FANCY_BORDER)
{
	SetLabel("Data Entry");
	fItem=NULL;
	fName=new wFieldControl(BRect(0,0,100,10),"data:entry:name","Name",table,"name",0,NULL);
	fName->SetDivider(75.0);
	fData=new BMenu("unknown data type");
	for (int32 *d=data_type_table;*d>-1;d++) {
		BMessage *msg=new BMessage(K_SELECT);
		msg->AddInt32("data_type",*d);
		fData->AddItem(new BMenuItem(name_of_data_type(*d),msg));
	}
	fData->SetLabelFromMarked(true);
	BMenuField *menuField=new BMenuField(BRect(0,0,300,10),"data:entry:type","Type",fData);
	menuField->SetDivider(75.0);
	wFieldControl *data_string=new wFieldControl(BRect(0,0,100,10),"_data_","Data :",Table(),"data",0,NULL);
	data_string->SetDivider(75.0);
	// ---
	PackChild(fName,K_TOP,K_LEFT,K_FILL_X);
	PackChild(menuField,K_TOP,K_LEFT,K_FILL_NONE);
	PackChild(data_string,K_TOP,K_LEFT,K_FILL_X);
}

void wDataEditor::AttachedToWindow(bool first_time)
{
	if (first_time) {
		fData->SetTargetForItems(this);
	}
}

void wDataEditor::MessageReceived(BMessage *msg)
{
	switch (msg->what) {
		case K_SELECT:{
			BMessage *rec=Table()->CurrentDataRecord();
			if (rec!=NULL) {
				int32 count=rec->FindInt32("countItems");
				if (count==0) {
					int32 type=msg->FindInt32("data_type");
					rec->ReplaceInt32("type",0,type);
					// ---
					/*
  					BMessage update(K_UPDATE); 
  					update.AddString("field_name","type");
  					update.AddInt32("field_index",0);
  					update.AddInt32("field_value",type);
  					SendToTable(&update);
  					*/
  					SendUpdate("type",0);
				} else {
					(new BAlert("error","Cannot change data-type of existing message entry","OK"))->Go();
					UpdateCurrentRecord(false);
				}
			}
			break;}
		default:
			inherited::MessageReceived(msg);
	}
}

void wDataEditor::UpdateCurrentRecord(bool selectionChanged)
{
	if (Table()->CurrentDataRecord()!=NULL) {
		int32 type=Table()->CurrentDataRecord()->FindInt32("type");
		// menu ----
		for (int32 i=fData->CountItems()-1;i>=0;i--) {
			BMessage *msg=fData->ItemAt(i)->Message();
			if (msg->FindInt32("data_type")==type)
				fData->ItemAt(i)->SetMarked(true);
		}
	}
}

// ----------------------------------------------------------------

wMessageDataViewer::wMessageDataViewer(BRect frame, const char *name,
	wTable *table,
	bool showSpecifiers,
	uint32 resizingMode, uint32 flags):
inherited(frame,name,B_SINGLE_SELECTION_LIST,resizingMode,flags),
wDBWidget(table)
{
	fShowSpecifiers=showSpecifiers;
}

void wMessageDataViewer::MessageReceived(BMessage *message)
{
	switch (message->what) {
		case K_UPDATE:{
			InvalidateItem(IndexOf(CurrentPropertyItem(0)));
			InvalidateItem(CurrentSelection(0));
			break;}
		default:
			inherited::MessageReceived(message);
	}
}
			

void wMessageDataViewer::DisplayMessageData(BMessage *msg)
{
	MakeEmpty();
	// -----
	fMessage=BMessage(msg);
	// -----
	char  *name; 
   	type_code  type; 
   	int32   count,j; 
   	void *data;
   	int32 size;
	char buf[128];
    wPropertyItem *item;
    BList *name_list=new BList();
    char *sdata;
   	for ( int32 i = 0;msg->GetInfo(B_ANY_TYPE, i, &name, &type, &count)==B_OK;i++ ) { 
   		if (fShowSpecifiers||// test if data is a specifiers prop
   			(strcmp(name,"specifiers")!=0&&strcmp(name,"_cur_specifier_")!=0)) {
   			AddItem(item=new wPropertyItem(&fMessage,name,type));
   			for (count--;count>=0;count--) {
   				AddUnder(new wDataItem(item,count),item);
   			}
   		}
  	}
  	delete name_list;
}

const BMessage *wMessageDataViewer::CurrentMessage() {return &fMessage;}

bool wMessageDataViewer::InitiateDrag(BPoint point, int32 index, bool wasSelected) 
{
	BListItem *item=ItemAt(index);
	wDataItem *data_item=dynamic_cast<wDataItem*>(item);
	int32 num; 
    Window()->CurrentMessage()->FindInt32("clicks", &num); 
	if (data_item!=NULL&&wasSelected&&num==1)
		return data_item->InitiateDrag(this,ItemFrame(index));
	else
		return false;
}
	
void wMessageDataViewer::MimicMessage(BMessage *msg)
{
	BListItem *item;
	wDataItem *data_item;
	status_t err;
	for (int32 count=FullListCountItems(),i=0;count>0;i++,count--) {
		item=FullListItemAt(i);
		data_item=dynamic_cast<wDataItem*>(item);
		if (data_item!=NULL)
			err=data_item->MimicMessage(msg);
	}
}

wPropertyItem *wMessageDataViewer::CurrentPropertyItem(int32 index)
{
	BListItem *item=ItemAt(CurrentSelection(index));
	wDataItem *data_item=dynamic_cast<wDataItem*>(item);
	if (data_item!=NULL) {
		return data_item->PropertyItem();
	} else {
		wPropertyItem *prop_item=dynamic_cast<wPropertyItem*>(item);
		return prop_item;
	}
}

wDataItem *wMessageDataViewer::CurrentDataItem(int32 index)
{
	BListItem *item=ItemAt(CurrentSelection(index));
	return (dynamic_cast<wDataItem*>(item));
}

void wMessageDataViewer::SelectionChanged(void)
{
	//SelectRecord(CurrentPropertyItem(0));
	BListItem *item=ItemAt(CurrentSelection(0));
	wDataItem *data_item=dynamic_cast<wDataItem*>(item);
	if (data_item!=NULL) {
		SelectRecord(data_item);
	} else {
		wPropertyItem *prop_item=dynamic_cast<wPropertyItem*>(item);
		SelectRecord(prop_item);
	}
}

void wMessageDataViewer::UpdateCurrentRecord(bool selectionChanged)
{
	if (!selectionChanged)
		InvalidateItem(IndexOf(CurrentPropertyItem(0)));
}

// ----------------------------------------------------------------
// ----------------------------------------------------------------

wMessageDataEditor::wMessageDataEditor(BRect frame, const char *name,bool showSpecifiers,
	//bool propagate,
	uint32 resizingMode, uint32 flags):
inherited(frame,name,resizingMode,flags)
{
	fDataTable=new wTable(name);
	fMessageDataViewer=new wMessageDataViewer(BRect(0,0,200,100),"datas",fDataTable,showSpecifiers);
	fDataEditor=new wDataEditor(BRect(0,0,100,100),"_data_editor_",fDataTable);
	// ---------
	fColorEditor=NULL;
	fRefEditor=NULL;
	fMessageEditor=NULL;
	// ---------
}

wMessageDataEditor::~wMessageDataEditor()
{
	fDataTable->CloseTable();
	//delete fColorEditor;
	//delete fRefEditor;
	//delete fMessageEditor;
}

void wMessageDataEditor::AttachedToWindow(bool first_time)
{
	if (first_time) {
		KPackView *buttons_panel=new KPackView(BRect(0,0,10,10),"_buttons_panel_",B_FOLLOW_ALL,B_FULL_UPDATE_ON_RESIZE,B_FANCY_BORDER);
		BButton *b;
		buttons_panel->PackChild(b=new BButton(BRect(0,0,50,20),"_add_","Add",new BMessage(K_ADD)),K_LEFT,K_TOP);
		b->SetTarget(this);
		buttons_panel->PackChild(b=new BButton(BRect(0,0,50,20),"_remove_","Remove",new BMessage(K_REMOVE)),K_LEFT,K_TOP);
		b->SetTarget(this);
		buttons_panel->PackChild(b=new BButton(BRect(0,0,50,20),"_edit_","Edit",new BMessage(K_INVOKE)),K_LEFT,K_TOP);
		b->SetTarget(this);
		// ---
		PackChild(add_scrollview(fMessageDataViewer,false,true),K_LEFT,K_TOP,K_FILL_Y,BPoint(2,2));
		Split();
		PackChild(buttons_panel,K_BOTTOM,K_LEFT,K_FILL_X,BPoint(2,2));
		PackChild(fDataEditor,K_FULL,K_LEFT,K_FILL_BOTH,BPoint(2,2));
		fMessageDataViewer->SetInvocationMessage(new BMessage(K_INVOKE));
		fMessageDataViewer->SetTarget(this);
	}
}

void wMessageDataEditor::SetMessage(BMessage *msg)
{
	if (msg!=NULL) {
		fMessageDataViewer->DisplayMessageData(msg);
	}
}

void wMessageDataEditor::MessageReceived(BMessage *msg)
{
	switch (msg->what) {
		case K_INVOKE:
			InvokeItem(fMessageDataViewer->CurrentPropertyItem(0),fMessageDataViewer->CurrentDataItem(0));
			break;
		case K_ADD:
			AddItem(fMessageDataViewer->CurrentPropertyItem(0));
			break;
		case K_REMOVE:
			RemoveItem(fMessageDataViewer->CurrentPropertyItem(0),fMessageDataViewer->CurrentDataItem(0));
			break;
		default:
			inherited::MessageReceived(msg);
	}
}

void wMessageDataEditor::InvokeItem(wPropertyItem *prop,wDataItem *data)
{
	BMessage extract;
	if (data!=NULL&&!data->IsLocked()) {
		char title[500];
		sprintf(title,"[%s]%s#%d",Window()->Title(),prop->Name(),data->Index());
		switch (prop->Type()) {
			case B_MESSAGE_TYPE:
				/*
				if (fMessageEditor==NULL)
					fMessageEditor=new wMessageEditorDialog(Window(),fDataTable,"data",0);
				fMessageEditor->Go(title);
				*/
				(new wMessageEditorDialog(Window(),fDataTable,"data",0))->Go(title);
				break;
			case B_RGB_COLOR_TYPE:
				if (fColorEditor==NULL)
					fColorEditor=new wColorEditorDialog(Window(),fDataTable,"data",0);
				fColorEditor->Go(title);
				break;
			case B_REF_TYPE:
				if (fRefEditor==NULL)
					fRefEditor=new wRefEditorDialog(Window(),fDataTable,"data",0);
				fRefEditor->Go(title);
				break;
		}
	}
}

void wMessageDataEditor::AddItem(wPropertyItem *prop)
{
	if (prop!=NULL) {
		wDataItem *data=new wDataItem(prop,prop->CountItems());
		fMessageDataViewer->AddUnder(data,prop);
	} else {
		prop=new wPropertyItem(fMessageDataViewer->CurrentMessage(),"???",0);
		fMessageDataViewer->AddItem(prop);
	}
}

void wMessageDataEditor::RemoveItem(wPropertyItem *prop,wDataItem *data)
{
	if (data!=NULL) {
		fDataTable->RemoveRecord(data);
		fMessageDataViewer->RemoveItem(data);
		delete data;
	} else if (prop!=NULL) {
		BListItem *item;
		while((item=fMessageDataViewer->ItemUnderAt(prop,true,0))!=NULL) {
			fDataTable->RemoveRecord(dynamic_cast<wRecord *>(item));
			fMessageDataViewer->RemoveItem(item);
			delete item;
		}
		fDataTable->RemoveRecord(prop);
		fMessageDataViewer->RemoveItem(prop);
		delete prop;
	}
}
	
void wMessageDataEditor::MimicMessage(BMessage *msg)
{
	fMessageDataViewer->MimicMessage(msg);
}
