//--------------------------------------------------------------------------
//
//			by Serge Fantino
//			fantino@mad.unice.fr
//
//		Copyright 1998, All Rights Reserved.
//
//--------------------------------------------------------------------------

#ifndef DATA_EDITOR_KMSF
#define DATA_EDITOR_KMSF

#include <InterfaceKit.h>

#include "pack_view.h"
#include "wDBObjects.h"

#include "wSpecialisedEditor.h"

class wPropertyItem : public BStringItem, public wRecord{
public:
	wPropertyItem(BMessage *msg,const char *name,type_code type);
	~wPropertyItem();
	
	void UpdateText();
	
	const char *OriginalName();
	const char *Name();
	void SetName(const char *name);
	type_code Type();
	BMessage *Message();
	int32 CountItems();
	
	status_t ExtractMessage(BMessage *result,int32 index);
	status_t ExtractData(void **data,int32 index,ssize_t *size);
	status_t MimicMessage(BMessage *mimic,int32 index);
	
	int32 AddItem(void *item);
	
	virtual void GetData(BMessage *message);
	virtual void SetData(BMessage *message,const char *fieldName,int32 fieldIndex);
	virtual const char *RecordName();
	
private:
	BMessage	*fMessage;
	char 		*fName,*fOriginalName;
	type_code	fType;
	int32		fCountItems;
};

class wDataItem : public BStringItem, public wRecord {
public:
	wDataItem(wPropertyItem *propertyItem,int32 index);
	
	void UpdateText();
	
	int32 Index();
	wPropertyItem *PropertyItem();
	
	status_t ExtractMessage(BMessage *result);
	status_t ExtractData(void **data,ssize_t *size);
	
	bool InitiateDrag(BView *view,BRect frame);
	
	status_t MimicMessage(BMessage *mimic);
	
	virtual void GetData(BMessage *message);
	virtual void SetData(BMessage *message,const char *fieldName,int32 fieldIndex);
	virtual const char *RecordName();
	
private:
	char			*fName;
	int32			fIndex;
	type_code		fType;
	wPropertyItem	*fPropertyItem;
};

class wDataEditor : public wDBWidget, public KPackView {
public:
	wDataEditor(BRect frame, const char *name, 
		wTable	*table,
		//bool propagate=true,
		uint32 resizingMode=B_FOLLOW_ALL_SIDES, 
		uint32 flags=0);
 	virtual void AttachedToWindow(bool first_time);
 	virtual void MessageReceived(BMessage *msg);
 	
 	virtual void UpdateCurrentRecord(bool selectionChanged);

private:
	typedef KPackView inherited;
	
	wPropertyItem	*fProp;
	BMessage		*fItem;
	
	wFieldControl	*fName;
	BMenu			*fData;
};

class wMessageDataViewer : public BOutlineListView, virtual public wDBWidget {
public:
	wMessageDataViewer(BRect frame, const char *name, 
		wTable *table,
		bool showSpecifiers=false,
		uint32 resizingMode = B_FOLLOW_LEFT | B_FOLLOW_TOP, 
		uint32 flags = B_WILL_DRAW | B_FRAME_EVENTS | B_NAVIGABLE);
	
	void MessageReceived(BMessage *m);
	
	wPropertyItem *CurrentPropertyItem(int32 index);
	wDataItem *CurrentDataItem(int32 index);
	virtual void SelectionChanged(void);
	virtual void UpdateCurrentRecord(bool selectionChanged);

	void DisplayMessageData(BMessage *msg);
	const BMessage *CurrentMessage();
	virtual bool InitiateDrag(BPoint point, int32 index, bool wasSelected);
	
	void MimicMessage(BMessage *msg);
private:
	typedef BOutlineListView inherited;
	BMessage 	fMessage;
	bool		fShowSpecifiers;
}; 

class wMessageDataEditor : public KPackView {
public:
	wMessageDataEditor(BRect frame, const char *name, 
		bool showSpecifiers=false,
		//bool propagate=true,
		uint32 resizingMode=B_FOLLOW_ALL_SIDES, 
		uint32 flags=0);
	~wMessageDataEditor();
		
 	virtual void AttachedToWindow(bool first_time);
	
	void SetMessage(BMessage *msg);
	
	virtual void MessageReceived(BMessage *msg);
	virtual void InvokeItem(wPropertyItem *prop,wDataItem *data);
	virtual void AddItem(wPropertyItem *prop);
	virtual void RemoveItem(wPropertyItem *prop,wDataItem *data);
	
	void MimicMessage(BMessage *msg);
private:
	typedef KPackView inherited;
	
	wTable					*fDataTable;
	wMessageDataViewer		*fMessageDataViewer;
	wDataEditor				*fDataEditor;
	
	wColorEditorDialog		*fColorEditor;
	wRefEditorDialog		*fRefEditor;
	wMessageEditorDialog	*fMessageEditor;
}; 

#endif
