//--------------------------------------------------------------------------
//
//			by Serge Fantino
//			fantino@mad.unice.fr
//
//		Copyright 1998, All Rights Reserved.
//
//--------------------------------------------------------------------------

#include <InterfaceKit.h>
#include <StorageKit.h>

#include "wMessageBuilder.h"

#include "constant.h"
#include "define.h"

#include "wApp.h"
#include "me_app.h"

// - LM
#include "tab_view.h"


#include "wMessageUtils.h"
#include "wDBObjects.h"
#include "wCommandEditor.h"
#include "wDataEditor.h"
#include "wSpecifierEditor.h"

//#include "wSuiteViewer.h"

// ----------------------------------------------------------------
 
wMessageEditor::wMessageEditor(BRect frame, const char *name,
	//bool propagate,
	uint32 resizingMode, uint32 flags):
inherited(frame,name,resizingMode,flags)
{
	fCommandEditor=new wCommandEditor(BRect(0,0,50,20),"Command",B_FOLLOW_ALL,B_WILL_DRAW|B_FULL_UPDATE_ON_RESIZE);
	fMessageDataEditor=new wMessageDataEditor(BRect(0,0,50,100),"Datas",true);
	fMessageSpecifierEditor=new wMessageSpecifierEditor(BRect(0,0,10,100),"Specifiers");
	fMessageInfoViewer=new wMessageInfoViewer(BRect(0,0,50,100),"Info");
	fTabView=new KTabView(BRect(0,0,50,50),"_message_info_",B_WIDTH_AS_USUAL,B_FOLLOW_ALL);
	// command -----------
	PackChild(fCommandEditor,K_TOP,K_LEFT,K_FILL_X,BPoint(2,2));
	// tab ---------------
	PackChild(fTabView,K_FULL,K_TOP,K_FILL_BOTH);
	// --- data editor
	fTabView->AddTab(fMessageDataEditor);
	// --- specifier editor
	fTabView->AddTab(fMessageSpecifierEditor);
	// --- info viewer
	fTabView->AddTab(fMessageInfoViewer);
}
	
/*
void wMessageEditor::AttachedToWindow()
{
}
*/

void wMessageEditor::AllAttached()
{
	inherited::AllAttached();
	/*
	// --- data editor
	fTabView->AddTab(fMessageDataEditor);
	// --- specifier editor
	fTabView->AddTab(fMessageSpecifierEditor);
	// --- info viewer
	fTabView->AddTab(fMessageInfoViewer);
	*/
	// --- messenger viewer
    /*
	wSimplePacker *panel;
    panel=new wSimplePacker(BRect(0,0,50,50),"messenger",false);
    fSuiteViewer=NULL;
    BMessage archive(B_ARCHIVED_OBJECT);
    archive.AddString("class","wSuiteViewer");
    archive.AddString("add_on",SUITE_EXPLORER_MIME_TYPE);
    archive.AddRect("_frame",BRect(0,0,100,100));
   	archive.AddInt32("_flags",B_WILL_DRAW);
    BArchivable *unarchived = instantiate_object(&archive); 
   	if ( unarchived ) { 
       	fSuiteViewer = cast_as(unarchived, wSuiteViewer); 
       	if ( fSuiteViewer ) { 
       		panel->PackChild(add_scrollview(fSuiteViewer,true,true),K_TOP,K_TOP,K_FILL_BOTH,true);
       	}
    }
    //fSuiteViewer=new wSuiteViewer(BRect(0,0,100,100),"_messenger_viewer_");
    //panel->PackChild(add_scrollview(fSuiteViewer,true,true),K_TOP,K_TOP,K_FILL_BOTH,true);
	fTabView->AddTab(panel);
    */
	// ---
    //fTabView->Select(0);
}

void wMessageEditor::SetMessage(BMessage *msg)
{
	if (msg!=NULL) {
		fMessageInfoViewer->SetMessage(msg);
		fCommandEditor->SetCommand(msg->what);
		fMessageDataEditor->SetMessage(msg);
		fMessageSpecifierEditor->SetMessage(msg);
	}
}

void wMessageEditor::MimicMessage(BMessage *msg)
{
	fCommandEditor->MimicMessage(msg);
	fMessageDataEditor->MimicMessage(msg);
	fMessageSpecifierEditor->MimicMessage(msg);
}

void wMessageEditor::SetMessenger(BMessenger messenger)
{
	fMessenger=messenger;
/*
	if (fSuiteViewer!=NULL) {
		if (!messenger.IsTargetLocal())
			fSuiteViewer->DisplayMessenger(messenger);
		else fSuiteViewer->MakeEmpty();
	}
*/
}

const BMessenger wMessageEditor::Messenger()
{
	return fMessenger;
}

bool wMessageEditor::Modified()
{
	return false;//
}

// ---------------------------------------------------------------------

#define BUTTON_FLAGS (B_WILL_DRAW | B_NAVIGABLE | B_FULL_UPDATE_ON_RESIZE)

class wSpecialButton : public BButton {
public:
	wSpecialButton(BRect frame,const char *name,const char *label,
		BMessage *msg,
		uint32 resizingMode = B_FOLLOW_LEFT | B_FOLLOW_TOP, 
        uint32 flags = BUTTON_FLAGS):
	BButton(frame,name,label,msg,resizingMode,flags)
	{}
	
	virtual void MouseDown(BPoint point)
	{
		Invoke();
	}
};

wMessageBuilder::wMessageBuilder(BRect frame, const char *title,
	BMessage *msg,
	BMessenger sendAddress,
	BHandler *replyAddress,
	uint32 workspaces):
inherited(frame,title,B_TITLED_WINDOW,0,workspaces)
{
	fSendAddress=sendAddress;
	fReplyAddress=replyAddress;
	// ---
	fFile=NULL;
	// ---
	Init(msg);
}

wMessageBuilder::wMessageBuilder(BRect frame,entry_ref *ref,BHandler *replyAddress):
inherited(frame,ref->name,B_TITLED_WINDOW,0,B_CURRENT_WORKSPACE)
{
	fReplyAddress=replyAddress;
	// ---
	fRef=*ref;
	fFile=new BFile(ref,B_READ_WRITE);
	BMessage unflatten;
	unflatten.Unflatten(fFile);
	// ---
	fSendAddress=unflatten.ReturnAddress();
	Init(&unflatten);
}

void wMessageBuilder::Init(BMessage *msg)
{
	fSelfMessenger=BMessenger(this);
	fSavePanel=NULL;
	// ---
	KPackView *v=new KPackView(Bounds(),"_main_");
	// --- menu
	BMenuBar *mb=new BMenuBar(BRect(0,0,20,10),"_menu_bar_");
	BMenu *m=new BMenu("Message");
	BMenuItem *item;
	BMessage *invoke;
	m->AddItem(new BMenuItem("Send message",invoke=new BMessage(K_ACTION),'R'));
	invoke->AddInt32("action",K_ACTION_SEND);
	m->AddItem(new BMenuItem("Instanciate message",invoke=new BMessage(K_ACTION),'I'));
	invoke->AddInt32("action",K_ACTION_INSTANCIATE);
	m->AddItem(new BSeparatorItem());
	m->AddItem(new BMenuItem("New message",invoke=new BMessage(K_ACTION),'N'));
	invoke->AddInt32("action",K_ACTION_NEW);
	m->AddItem(new BMenuItem("Copy message",invoke=new BMessage(K_ACTION),'D'));
	invoke->AddInt32("action",K_ACTION_COPY);
	m->AddItem(new BSeparatorItem());
	m->AddItem(new BMenuItem("Save message",invoke=new BMessage(K_ACTION),'S'));
	invoke->AddInt32("action",K_ACTION_SAVE);
	m->AddItem(new BMenuItem("Save message as...",invoke=new BMessage(K_ACTION),'S',B_SHIFT_KEY));
	invoke->AddInt32("action",K_ACTION_SAVE_AS);
	m->AddItem(new BSeparatorItem());
	m->AddItem(new BMenuItem("Close",new BMessage(B_QUIT_REQUESTED),'W'));
	m->SetTargetForItems(this);
	m->AddItem(item=new BMenuItem("Quit",new BMessage(B_QUIT_REQUESTED),'Q'));
	item->SetTarget(be_app);
	mb->AddItem(m);
	// --- buttons
	KPackView *vb=new KPackView(BRect(0,0,30,20),"_buttons_",B_FOLLOW_ALL);//,B_FULL_UPDATE_ON_RESIZE,B_FANCY_BORDER);
	BButton	*b;
	b=new wSpecialButton(BRect(0,0,10,20),"_dragger_","drag message",invoke=new BMessage(K_ACTION));
	//tips->AddTips("drag the message you're currently editing to any target",b);
	invoke->AddInt32("action",K_ACTION_DRAG);
	b->SetTarget(this);
	vb->PackChild(b,K_LEFT,K_TOP,K_FILL_NONE,BPoint(2,2));
	b=new wSpecialButton(BRect(0,0,10,20),"_sender_","send message",invoke=new BMessage(K_ACTION));
	invoke->AddInt32("action",K_ACTION_SEND);
	b->SetTarget(this);
	vb->PackChild(b,K_LEFT,K_TOP,K_FILL_NONE,BPoint(0,2));
	b=new wSpecialButton(BRect(0,0,10,20),"_dropper_","drop messenger",invoke=new BMessage(K_ACTION));
	invoke->AddInt32("action",K_ACTION_SET);
	b->SetTarget(this);
	vb->PackChild(b,K_LEFT,K_TOP,K_FILL_NONE,BPoint(2,2));
	// ---
	fEditor=new wMessageEditor(BRect(0,0,10,10),"v_editor");
	// ---
	v->PackChild(mb,K_TOP,K_LEFT,K_FILL_X,BPoint(-1,0));
	v->PackChild(vb,K_TOP,K_LEFT,K_FILL_X);
	v->PackChild(fEditor,K_FULL,K_LEFT,K_FILL_BOTH);
	AddChild(v);
	if (msg!=NULL) fEditor->SetMessage(msg);
	fEditor->SetMessenger(fSendAddress);
}

wMessageBuilder::~wMessageBuilder()
{
	if (fSavePanel!=NULL) delete fSavePanel;
	delete fFile;
}

void wMessageBuilder::MessageReceived(BMessage *msg)
{
	if (msg->WasDropped()) {
		if (msg->what==B_SIMPLE_DATA) {
			BMessenger messenger;
			if (msg->FindMessenger("messenger",&messenger)==B_OK)
				fEditor->SetMessenger(messenger);
		} else
			BWindow::MessageReceived(msg);
	} else
	switch (msg->what) {
		case K_ACTION:{
			int32 action=msg->FindInt32("action");
			switch (action) {
		case K_ACTION_DRAG:{
			BMessage drag;
			fEditor->MimicMessage(&drag);
			fEditor->DragMessage(&drag,BRect(10,10,50,50),fReplyAddress);
			break;}
		case K_ACTION_SEND:
			if (fSendAddress.IsValid()) {
				BMessage send;
				fEditor->MimicMessage(&send);
				if ((fEditor->Messenger()).SendMessage(&send,fReplyAddress)!=B_OK)
					(new BAlert("error","error when sending message","OK"))->Show();
			} else
				(new BAlert("error","messenger is no longer valid","OK"))->Show();
			break;
		case K_ACTION_SET:
			(new BAlert("info","Drop a messenger here","OK"))->Show();
			break;
		case K_ACTION_NEW:
			ME_app->OpenMessage("New Message",ME_app->GetNewIndex(),new BMessage(),(fEditor->Messenger()));
			break;
		case K_ACTION_COPY:{
			BMessage copy;
			fEditor->MimicMessage(&copy);
			ME_app->OpenMessage("Copy Message",ME_app->GetNewIndex(),&copy,(fEditor->Messenger()));
			break;}
		case K_ACTION_SAVE:
			if (fFile!=NULL) {
				// Flatten the message
				BMessage flat;
				ssize_t size;
				fEditor->MimicMessage(&flat);
				fFile->Seek(0,SEEK_SET);
				flat.Flatten(fFile,&size);
				break;
			}
			// else continue to save_as
		case K_ACTION_SAVE_AS:
			if (fSavePanel==NULL) {
				fSavePanel=new BFilePanel(
					B_SAVE_PANEL,&fSelfMessenger, NULL, 0, false,
					new BMessage(B_SAVE_REQUESTED), NULL, false, true);
				fSavePanel->Window()->SetTitle(Title());
			}
			fSavePanel->Show();
			break;
		case K_ACTION_INSTANCIATE:{
			BMessage archive;
			fEditor->MimicMessage(&archive);
			BArchivable *unarchived = instantiate_object(&archive); 
   			if ( unarchived ) { 
       			BView *view = cast_as(unarchived, BView); 
       			if ( view ) { 
           			BWindow *w=new BWindow(view->Frame(),view->Name(),B_TITLED_WINDOW,0);
           			w->AddChild(view);
           			w->Show();
       			} else {
       				delete unarchived;
       				(new BAlert("error","unarchive succeded but it's not a view...\n(so I just delete it...)","OK"))->Show();
   				}
   			} else
       			(new BAlert("error","unarchive failed","OK"))->Show();
			break;}
		}
		break;
		}
		case B_SAVE_REQUESTED:{
		// from John Cooke
		// extract directory
			entry_ref ref;
			msg->FindRef("directory", &ref);
			BDirectory dir;
			dir.SetTo(&ref);
		// extract file name
			const char *name;
			msg->FindString("name", &name);
		// create new file
			if (fFile!=NULL) delete fFile;
			fFile = new BFile(&dir,name,B_READ_WRITE|B_CREATE_FILE);
			//dir.CreateFile(name, newfile);
		// set MIME type
			BNodeInfo noddy (fFile);
			noddy.SetType(MY_APP_MESSAGE_SIGN);
		// Flatten the message
			BMessage flat;
			ssize_t size;
			fEditor->MimicMessage(&flat);
			flat.Flatten(fFile,&size);
			SetTitle(name);
		}
			break;
		default:
			inherited::MessageReceived(msg);
	}
}
 	