//--------------------------------------------------------------------------
//
//			by Serge Fantino
//			fantino@math.unice.fr
//
//		Copyright 1998, All Rights Reserved.
//
//--------------------------------------------------------------------------

#include "wMessageUtils.h"

#include "constant.h"

#include <StorageKit.h>
#include <AppKit.h>
#include <InterfaceKit.h>
	
//   _________________________
// _| Supplementary Functions |_______________________________________________
//
#include <stdio.h>
#include <string.h>
#include <ctype.h>


int32 data_type_table[]={
	B_BOOL_TYPE,
	B_CHAR_TYPE,
	B_COLOR_8_BIT_TYPE,
	B_DOUBLE_TYPE,
	B_FLOAT_TYPE,
	B_GRAYSCALE_8_BIT_TYPE,
	B_INT64_TYPE,
	B_INT32_TYPE,
	B_INT16_TYPE,
	B_INT8_TYPE,
	B_MESSAGE_TYPE,
	B_MESSENGER_TYPE,
	B_MIME_TYPE,
	B_MONOCHROME_1_BIT_TYPE,
	B_OBJECT_TYPE,
	B_OFF_T_TYPE,
	B_PATTERN_TYPE,
	B_POINTER_TYPE,
	B_POINT_TYPE,
	B_RAW_TYPE,
	B_RECT_TYPE,
	B_REF_TYPE,
	B_RGB_32_BIT_TYPE,
	B_RGB_COLOR_TYPE,
	B_SIZE_T_TYPE,
	B_SSIZE_T_TYPE,
	B_STRING_TYPE,
	B_TIME_TYPE,
	B_UINT64_TYPE,
	B_UINT32_TYPE,
	B_UINT16_TYPE,
	B_UINT8_TYPE,
	-1,
	};

int32 specifier_table[]={
	B_NO_SPECIFIER,
	B_DIRECT_SPECIFIER,
	B_INDEX_SPECIFIER,
	B_REVERSE_INDEX_SPECIFIER,
	B_RANGE_SPECIFIER,
	B_REVERSE_RANGE_SPECIFIER,
	B_NAME_SPECIFIER,
	B_ID_SPECIFIER,
	-1
	};

//--[convert a uint32 into 4 ASCII chars or 8 hex digits]--
// only if all bytes are printable will the former apply
// (from john cooke ------ )
void
seewhat (
	uint32 in,
	char *out	// must be 9 chars at least
)
{
	char buf4[4];
	char buf8[8];
	uint8 *ptr = (uint8 *)&in;
	bool ean=true;
	static char *hex = "0123456789abcdef";

//--[load both char arrays]--
	for(int i=0; i<4; i++,ptr++)
	{
		if(ean && isprint(*ptr))
			buf4[i] = (char)*ptr;
		else
			ean = false;

		uint8 nibble = *ptr >> 4;
		buf8[i*2] = hex[nibble];
		nibble = *ptr & 15;
		buf8[i*2+1] = hex[nibble];
	}

//--[copy output]--
	if(ean)
	{
		memcpy(out, buf4, 4);
		out[4] = '\0';
	}
	else
	{
		memcpy(out, buf8, 8);
		out[8] = '\0';
	}
};

const char *name_of_specifier(int32 specifier) 
{
	switch (specifier) {
		case B_NO_SPECIFIER:return "B_NO_SPECIFIER";
		case B_DIRECT_SPECIFIER:return "B_DIRECT_SPECIFIER";
		case B_INDEX_SPECIFIER:return "B_INDEX_SPECIFIER";
		case B_REVERSE_INDEX_SPECIFIER:return "B_REVERSE_INDEX_SPECIFIER";
		case B_RANGE_SPECIFIER:return "B_RANGE_SPECIFIER";
		case B_REVERSE_RANGE_SPECIFIER:return "B_REVERSE_RANGE_SPECIFIER";
		case B_NAME_SPECIFIER:return "B_NAME_SPECIFIER";
		case B_ID_SPECIFIER:return "B_ID_SPECIFIER";
		default:return "???";
	}
};

const char *name_of_command(int32 command) 
{
	switch (command) {
		case B_GET_PROPERTY:return "B_GET_PROPERTY";
		case B_SET_PROPERTY:return "B_SET_PROPERTY";
		case B_EXECUTE_PROPERTY:return "B_EXECUTE_PROPERTY";
		case B_COUNT_PROPERTIES:return "B_COUNT_PROPERTIES";
		case B_CREATE_PROPERTY:return "B_CREATE_PROPERTY";
		case B_DELETE_PROPERTY:return "B_DELETE_PROPERTY";
		case B_GET_SUPPORTED_SUITES:return "B_GET_SUPPORTED_SUITES";
		// ---
		case B_ABOUT_REQUESTED:return "B_ABOUT_REQUESTED";
		case B_QUIT_REQUESTED:return "B_QUIT_REQUESTED";
		// ---
		case B_SIMPLE_DATA:return "B_SIMPLE_DATA";
		case B_REFS_RECEIVED:return "B_REFS_RECEIVED";
		case B_ARCHIVED_OBJECT:return "B_ARCHIVED_OBJECT";
		case B_REPLY:return "B_REPLY";
		case B_MESSAGE_NOT_UNDERSTOOD:return "B_MESSAGE_NOT_UNDERSTOOD";
		default: return name_of_specifier(command);
	}
}

const char *name_of_data_type(type_code typeCode)
{
	switch (typeCode) {
		case B_BOOL_TYPE:return "B_BOOL_TYPE";
		case B_CHAR_TYPE:return "B_CHAR_TYPE";
		case B_COLOR_8_BIT_TYPE:return "B_COLOR_8_BIT_TYPE";
		case B_DOUBLE_TYPE:return "B_DOUBLE_TYPE";
		case B_FLOAT_TYPE:return "B_FLOAT_TYPE";
		case B_GRAYSCALE_8_BIT_TYPE:return "B_GRAYSCALE_8_BIT_TYPE";
		case B_INT64_TYPE:return "B_INT64_TYPE";
		case B_INT32_TYPE:return "B_INT32_TYPE";
		case B_INT16_TYPE:return "B_INT16_TYPE";
		case B_INT8_TYPE:return "B_INT8_TYPE";
		case B_MESSAGE_TYPE:return "B_MESSAGE_TYPE";
		case B_MESSENGER_TYPE:return "B_MESSENGER_TYPE";
		case B_MIME_TYPE:return "B_MIME_TYPE";
		case B_MONOCHROME_1_BIT_TYPE:return "B_MONOCHROME_1_BIT_TYPE";
		case B_OBJECT_TYPE:return "B_OBJECT_TYPE";
		case B_OFF_T_TYPE:return "B_OFF_T_TYPE";
		case B_PATTERN_TYPE:return "B_PATTERN_TYPE";
		case B_POINTER_TYPE:return "B_POINTER_TYPE";
		case B_POINT_TYPE:return "B_POINT_TYPE";
		case B_RAW_TYPE:return "B_RAW_TYPE";
		case B_RECT_TYPE:return "B_RECT_TYPE";
		case B_REF_TYPE:return "B_REF_TYPE";
		case B_RGB_32_BIT_TYPE:return "B_RGB_32_BIT_TYPE";
		case B_RGB_COLOR_TYPE:return "B_RGB_COLOR_TYPE";
		case B_SIZE_T_TYPE:return "B_SIZE_T_TYPE";
		case B_SSIZE_T_TYPE:return "B_SSIZE_T_TYPE";
		case B_STRING_TYPE:return "B_STRING_TYPE";
		case B_TIME_TYPE:return "B_TIME_TYPE";
		case B_UINT64_TYPE:return "B_UINT64_TYPE";
		case B_UINT32_TYPE:return "B_UINT32_TYPE";
		case B_UINT16_TYPE:return "B_UINT16_TYPE";
		case B_UINT8_TYPE:return "B_UINT8_TYPE";
		default:return "unknown data type";
	}
};

// (from john cooke ------ )
void sprint_data(char *buf,int32 type,BMessage *hmsg,const char *name,int32 j)
{
	BPoint point;
	BRect rect;
	switch(type)
			{
			case B_BOOL_TYPE:
				bool ean;
				hmsg->FindBool(name, j, &ean);
				sprintf(buf, "%s", ean?"true":"false");
				break;
			case B_INT8_TYPE:
				int8 i8;
				hmsg->FindInt8(name, j, &i8);
				sprintf(buf, "%d", i8);
				break;
			case B_INT16_TYPE:
				int16 i16;
				hmsg->FindInt16(name, j, &i16);
				sprintf(buf, "%d", i16);
				break;
			case B_INT32_TYPE:
				int32 i32;
				hmsg->FindInt32(name, j, &i32);
				sprintf(buf, "%d", i32);
				break;
			case B_INT64_TYPE:
				int64 i64;
				hmsg->FindInt64(name, j, &i64);
				sprintf(buf, "%ld", i64);
				break;
			case B_FLOAT_TYPE:
				float f;
				hmsg->FindFloat(name, j, &f);
				sprintf(buf, "%f", f);
				break;
			case B_DOUBLE_TYPE:
				double d;
				hmsg->FindDouble(name, j, &d);
				sprintf(buf, "%f", d);
				break;
			case B_STRING_TYPE:
				char * str;
				hmsg->FindString(name, j, &str);
				sprintf(buf, "[%.60s]", str);
				break;
			case B_POINT_TYPE:
				hmsg->FindPoint(name, j, &point);
				sprintf(buf, "%.1f,%.1f", point.x, point.y);
				break;
			case B_RECT_TYPE:
				hmsg->FindRect(name, j, &rect);
				sprintf(buf, "%.1f,%.1f,%.1f,%.1f", rect.left, rect.top, rect.right, rect.bottom);
				break;
			case B_RGB_COLOR_TYPE:
			{
				rgb_color *col;
				ssize_t nbyte;
				hmsg->FindData(name, type, j, (void **)&col, &nbyte);
				sprintf(buf, "R%d G%d B%d A%d", col->red, col->green, col->blue, col->alpha);
			}
				break;
			case B_REF_TYPE:
			{
				entry_ref ref;
				hmsg->FindRef(name, j, &ref);

				BEntry entry(&ref);
				BPath path;
				entry.GetPath(&path);
				sprintf(buf, "[%.60s]", path.Path());
			}
				break;
			case B_POINTER_TYPE:
			{
				void *ptr;
				ssize_t nbyte;
				hmsg->FindData(name, type, j, &ptr, &nbyte);
				sprintf(buf, "%p", ptr);
			}
				break;
			case B_MIME_TYPE:
			{
				void *ptr;
				ssize_t nbyte;
				hmsg->FindData(name, type, j, &ptr, &nbyte);
				sprintf(buf, "%.60s", (char *)ptr);
			}
				break;
			case B_MESSAGE_TYPE:
			{
				BMessage mess;
				hmsg->FindMessage(name,j,&mess);
				sprintf(buf,"%.60s",name_of_command(mess.what));
			}
				break;
			default:
				void *data;
				ssize_t nbyte;
				if (hmsg->FindData(name, type, j, &data, &nbyte)==B_OK)
					sprintf(buf, "%d bytes at %p", nbyte, data);
				else
					sprintf(buf,"");
				break;
	}
};


BMenuItem *new_command_item(int32 command) 
{
	BMessage *msg=new BMessage(K_SELECT);
	msg->AddInt32("command",command);
	return (new BMenuItem(name_of_command(command),msg));
};