//--------------------------------------------------------------------------
//
//			by Serge Fantino
//			fantino@mad.unice.fr
//
//		Copyright 1998, All Rights Reserved.
//
//--------------------------------------------------------------------------

#include <StorageKit.h>
#include <InterfaceKit.h>

#include "wSpecialisedEditor.h"

#include "constant.h"

// - LM
#include "pack_view.h"

#include "wMessageBuilder.h"

wSpecialisedEditorDialog::wSpecialisedEditorDialog(
		BWindow *owner,const char *name,
		wTable *table,const char *fieldName,int32 fieldIndex):
BHandler(name),
wDBFieldWidget(table,fieldName,fieldIndex)
{
	if (owner->Lock()) {
		owner->AddHandler(this);
		owner->Unlock();
	}
}

const BWindow *wSpecialisedEditorDialog::Owner()
{
	return (BWindow *)Looper();
}

void wSpecialisedEditorDialog::ClosingTable()
{
	delete this;
}

// --------------------------------------------------

class wDuplexMessage: public BMessageFilter, public BInvoker {
public:
	wDuplexMessage(int32 command,const BHandler *handler):
	BMessageFilter(command),
	BInvoker(NULL,handler)
	{
	}
	
	virtual filter_result Filter(BMessage *message, BHandler **target)
	{
		Invoke(message);
		return B_SKIP_MESSAGE;
	}
};

// ------------------

wColorEditorDialog::wColorEditorDialog(BWindow *owner,
		wTable *table,const char *fieldName,int32 fieldIndex):
wSpecialisedEditorDialog(owner,"color_editor",table,fieldName,fieldIndex)
{
  	fWindow=NULL;
  	fColorControl=NULL;
}


wColorEditorDialog::~wColorEditorDialog()
{
	if (fWindow!=NULL) fWindow->Quit();
}

BWindow *wColorEditorDialog::Go(const char *title)
{
	if (fWindow==NULL) {
		fColorControl=new
			BColorControl(BPoint(0,0),B_CELLS_16x16,10, 
      	   "_color_control_",new BMessage(K_SET));
  		fColorControl->SetTarget(this);
    	BRect frame=Owner()->Frame();
    	BRect rect=fColorControl->Bounds();
    	rect.OffsetTo(frame.right+9,frame.top+13);
   		fWindow=new BWindow(rect,title,
    		B_FLOATING_WINDOW_LOOK,B_FLOATING_SUBSET_WINDOW_FEEL,
    		B_NOT_RESIZABLE|B_NOT_ZOOMABLE);
    	fWindow->AddChild(fColorControl);
    	fWindow->AddToSubset(Owner());
    	fWindow->AddFilter(new wDuplexMessage(B_QUIT_REQUESTED,this));
    }
    fWindow->SetTitle(title);
    if (fWindow->IsHidden())
   		fWindow->Show();
	UpdateCurrentRecord(true);
    return fWindow;
}

void wColorEditorDialog::UpdateCurrentRecord(bool selectionChanged)
{
	inherited::UpdateCurrentRecord(selectionChanged);
	rgb_color *color;
	ssize_t size;
	if (fWindow!=NULL&&fWindow->Lock()) {
		if (Data()!=NULL&&Data()->FindData(FieldName(),B_RGB_COLOR_TYPE,(void **)&color,&size)==B_OK) {
			fColorControl->SetEnabled(true);
			fColorControl->SetValue(*color);
		} else
			fColorControl->SetEnabled(false);
		fWindow->Unlock();
	}
}
	
void wColorEditorDialog::MessageReceived(BMessage *message)
{
	switch (message->what) {
		case K_SET:
			rgb_color color=fColorControl->ValueAsColor();
			SetData(&color,sizeof(rgb_color));
			SendUpdate();
			break;
		case B_QUIT_REQUESTED:
			if (fWindow!=NULL) {
				fWindow->Hide();
			}
			break;
		default:
			BHandler::MessageReceived(message);
	}
}

// -----------------------------------

wRefEditorDialog::wRefEditorDialog(BWindow *owner,
		wTable *table,const char *fieldName,int32 fieldIndex):
wSpecialisedEditorDialog(owner,"ref_editor",table,fieldName,fieldIndex)
{
  	fFilePanel=NULL;
}

wRefEditorDialog::~wRefEditorDialog()
{
	if (fFilePanel!=NULL) delete fFilePanel;
}

BWindow *wRefEditorDialog::Go(const char *title)
{
	if (fFilePanel==NULL) {
		fFilePanel=new BFilePanel(
			B_OPEN_PANEL, 
         	&BMessenger(this),
         	NULL,
         	B_FILE_NODE|B_DIRECTORY_NODE,//uint32 node_flavors = 0, 
         	false,//bool allow_multiple_selection = true, 
         	&BMessage(K_SET),//BMessage *message = NULL, 
         	NULL,//BRefFilter *filter = NULL, 
         	true,//bool modal = false, 
         	true//bool hide_when_done = true)
         	);
    	fFilePanel->Window()->SetFeel(B_MODAL_SUBSET_WINDOW_FEEL);
    	fFilePanel->Window()->AddToSubset(Owner());
    }
    fFilePanel->Window()->SetTitle(title);
    fFilePanel->Show();
	UpdateCurrentRecord(true);
    return fFilePanel->Window();
}

void wRefEditorDialog::UpdateCurrentRecord(bool selectionChanged)
{
	inherited::UpdateCurrentRecord(selectionChanged);
	entry_ref ref;
	if (fFilePanel!=NULL&&Data()!=NULL&&Data()->FindRef(FieldName(),FieldIndex(),&ref)==B_OK) {
		BEntry entry(&ref);
		entry.GetParent(&entry);
		fFilePanel->SetPanelDirectory(&entry);
		BMessage select(B_SET_PROPERTY);
		select.AddSpecifier("Selection");
		select.AddRef("data",&ref);
		fFilePanel->Window()->PostMessage(&select,fFilePanel->Window()->FindView("PoseView"));
	}
}
	
void wRefEditorDialog::MessageReceived(BMessage *message)
{
	switch (message->what) {
		case K_SET:{
			entry_ref ref;
			if (message->FindRef("refs",0,&ref)==B_OK&&Data()!=NULL) {
				Data()->ReplaceRef(FieldName(),FieldIndex(),&ref);
				SendUpdate();
			}
			break;}
		default:
			wSpecialisedEditorDialog::MessageReceived(message);
	}
}

// -------------------------------------------------------------------------


wMessageEditorDialog::wMessageEditorDialog(BWindow *owner,
		wTable *table,const char *fieldName,int32 fieldIndex):
wSpecialisedEditorDialog(owner,"color_editor",table,fieldName,fieldIndex)
{
  	fWindow=NULL;
  	fMessageEditor=NULL;
}

wMessageEditorDialog::~wMessageEditorDialog()
{
	if (fWindow!=NULL) fWindow->Quit();
}

BWindow *wMessageEditorDialog::Go(const char *title)
{
	if (fWindow==NULL) {
		BRect rect=Owner()->Frame();
		rect.OffsetBy(19,19);
		fWindow=new BWindow(rect,title,
			B_TITLED_WINDOW,
			//B_TITLED_WINDOW_LOOK,B_FLOATING_SUBSET_WINDOW_FEEL,
			B_NOT_ZOOMABLE);
		KPackView *main=new KPackView(fWindow->Bounds(),"_main_");
		KPackView *bp=new KPackView(BRect(0,0,100,20),"_buttons_panel_",0,B_FULL_UPDATE_ON_RESIZE,B_FANCY_BORDER);
		BButton *b=new BButton(BRect(0,0,70,25),"_ok_","OK",new BMessage(K_COMMIT));
		b->MakeDefault;
		b->SetTarget(this);
		bp->PackChild(b,K_RIGHT,K_TOP);
		bp->PackChild(b=new BButton(BRect(0,0,70,25),"_cancel_","Cancel",new BMessage(K_CANCEL)),K_RIGHT,K_TOP);
		b->SetTarget(this);
		main->PackChild(bp,K_BOTTOM,K_LEFT,K_FILL_X);
		fMessageEditor=new wMessageEditor(BRect(0,0,200,200),"_editor_",false);
		main->PackChild(fMessageEditor,K_FULL,K_LEFT,K_FILL_BOTH);
		fWindow->AddChild(main);
		// ---
		fWindow->AddToSubset(Owner());
    	fWindow->AddFilter(new wDuplexMessage(B_QUIT_REQUESTED,this));
	}
	LockRecord(false);
	UpdateCurrentRecord(true);
	LockRecord(true);
	fWindow->SetTitle(title);
	fWindow->Show();
	return fWindow;
}

void wMessageEditorDialog::UpdateCurrentRecord(bool selectionChanged)
{
	inherited::UpdateCurrentRecord(selectionChanged);
	if (selectionChanged&&!IsRecordLocked()) {
		BMessage message;
		if (fMessageEditor!=NULL&&Data()!=NULL&&Data()->FindMessage(FieldName(),FieldIndex(),&message)==B_OK)
			fMessageEditor->SetMessage(&message);
	}
}

void wMessageEditorDialog::RemoveRecord(wRecord *record)
{
	inherited::RemoveRecord(record);
	if (!IsRecordLocked()) {// edited message is being removed...
		QuitMessage();
	}
}

void wMessageEditorDialog::MessageReceived(BMessage *message)
{
	switch (message->what) {
		case B_QUIT_REQUESTED:
			if (fMessageEditor->Modified()) {
				(new BAlert("Quit Requested","Do you want to commit the message ?","Cancel","Commit"))
					->Go(new BInvoker(new BMessage('alrt'),BMessenger(this)));
			} else
				QuitMessage();
			break;
		case 'alrt':{
			int32 button_index;
			message->FindInt32("which", &button_index);
			if (button_index==0) {
				QuitMessage();
				break;
			}
			}
		case K_COMMIT:
			SaveMessage();
			QuitMessage();
			break;
		case K_CANCEL:
			QuitMessage();
			break;
		default:
			inherited::MessageReceived(message);
	}
}

void wMessageEditorDialog::SaveMessage()
{
	BMessage result;
	fMessageEditor->MimicMessage(&result);
	SyncRecord();
	if (Data()!=NULL)
		Data()->ReplaceMessage(FieldName(),FieldIndex(),&result);
	SendUpdate();
}

void wMessageEditorDialog::QuitMessage()
{
	fWindow->Quit();
	fWindow=NULL;
	fMessageEditor=NULL;
	delete this;
}

