//--------------------------------------------------------------------------
//
//			by Serge Fantino
//			fantino@mad.unice.fr
//
//		Copyright 1998, All Rights Reserved.
//
//--------------------------------------------------------------------------

#include <stdio.h>

#include <AppKit.h>
#include <InterfaceKit.h>

#include "wSpecifierEditor.h"

#include "constant.h"

// - LM
#include "tab_view.h"
#include "scroll_view.h"

#include "wMessageUtils.h"
#include "wDBObjects.h"
#include "wCommandEditor.h"
#include "wDataEditor.h"

// ****************************************************************
//  SPECIFIER
// ****************************************************************

class wSpecifierItem : public BStringItem, public wRecord { // -----------------------------------
public:
	wSpecifierItem(BMessage *specifier,int32 what,const char *property):
	BStringItem(""),
	wRecord()
	{
		fWhat=what;
		fSpecifier=BMessage(specifier);
		if (!fSpecifier.HasString("property"))
			fSpecifier.AddString("property",property);
		UpdateText();
	}
	
	BMessage *Specifier() {return &fSpecifier;}
	
	void UpdateText()
	{
		char buf[200];
		switch (fSpecifier.what) {
			case B_DIRECT_SPECIFIER:
				sprintf(buf,"%s",fSpecifier.FindString("property"));
				break;
			case B_NAME_SPECIFIER:
				sprintf(buf,"%s [%s]",fSpecifier.FindString("property"),fSpecifier.FindString("name"));
				break;
			case B_ID_SPECIFIER:
				sprintf(buf,"%s [%d]",fSpecifier.FindString("property"),fSpecifier.FindInt32("id"));
				break;
			case B_INDEX_SPECIFIER:
				sprintf(buf,"%s [+%d]",fSpecifier.FindString("property"),fSpecifier.FindInt32("index"));
				break;
			case B_REVERSE_INDEX_SPECIFIER:
				sprintf(buf,"%s [-%d]",fSpecifier.FindString("property"),fSpecifier.FindInt32("index"));
				break;
			case B_RANGE_SPECIFIER:
				sprintf(buf,"%s [+%d-%d]",fSpecifier.FindString("property"),fSpecifier.FindInt32("index"),fSpecifier.FindInt32("range"));
				break;
			case B_REVERSE_RANGE_SPECIFIER:
				sprintf(buf,"%s [-%d-%d]",fSpecifier.FindString("property"),fSpecifier.FindInt32("index"),fSpecifier.FindInt32("range"));
				break;
			default:
				sprintf(buf,"%s [?]",fSpecifier.FindString("property"));
		}
		SetText(buf);
	}
	
	void GetData(BMessage *message)
	{
		*message=BMessage(fSpecifier);
		//message->what=B_DIRECT_SPECIFIER;
	}
	
	void SetData(BMessage *message,const char *fieldName,int32 fieldIndex)
	{
		fSpecifier=message;
		UpdateText();
	}
	
private:
	int32		fWhat;
	BMessage	fSpecifier;
};

// ---------------------------------------------------------

wSpecifierEditor::wSpecifierEditor(BRect frame, const char *name,wTable *table,uint32 resizingMode, uint32 flags):
inherited(frame,name,resizingMode,flags,B_FANCY_BORDER),
wDBWidget(table)
{
	// --- spec type
	fSpecMenu=new BMenu("m_spec_type");
	BuildSpecifierTypeMenu(fSpecMenu);
	fSpecMenu->SetLabelFromMarked(true);
	// --- property
	fPropertyName=new wFieldControl(BRect(0,0,100,20),"w_property_name","Property:",table,"property",0,NULL);
	// --- name value
	fNameValue=new wFieldControl(BRect(0,0,100,20),"w_name_value","Name:",table,"name",0,NULL);
}

void wSpecifierEditor::BuildSpecifierTypeMenu(BMenu *menu)
{
	for (int32 i=0;specifier_table[i]>-1;i++) {
		fSpecMenu->AddItem(new_command_item(specifier_table[i]));
	}
}

void wSpecifierEditor::AttachedToWindow(bool first_time)
{
	if (first_time) {
		fPropertyName->SetDivider(75.0);
		PackChild(fPropertyName,K_TOP,K_LEFT,K_FILL_X);
		// ---
		fSpecMenu->SetTargetForItems(this);
		BMenuField* menuField = 
			new BMenuField(BRect(0,0,300,20),"MenuField","Spec Type:", fSpecMenu);
		menuField->SetDivider(75.0);
		PackChild(menuField,K_TOP,K_LEFT,K_FILL_NONE);
		// ---
		fNameValue->SetDivider(75.0);
		PackChild(fNameValue,K_TOP,K_LEFT,K_FILL_X);
		// ---
		wFieldControl *wfc;
		wfc=new wFieldControl(BRect(0,0,100,20),"w_id_value","ID:",Table(),"id",0,NULL);
		wfc->SetDivider(75.0);
		PackChild(wfc,K_TOP,K_LEFT,K_FILL_X);
		wfc=new wFieldControl(BRect(0,0,100,20),"w_index_value","Index:",Table(),"index",0,NULL);
		wfc->SetDivider(75.0);
		PackChild(wfc,K_TOP,K_LEFT,K_FILL_X);
		wfc=new wFieldControl(BRect(0,0,100,20),"w_range_value","Range:",Table(),"range",0,NULL);
		wfc->SetDivider(75.0);
		PackChild(wfc,K_TOP,K_LEFT,K_FILL_X);
	}
}

void wSpecifierEditor::MessageReceived(BMessage *msg)
{
	int32 command;
	switch (msg->what) {
		case K_SELECT:
			if (msg->FindInt32("command",&command)==B_OK&&fSpecifier!=NULL) {
				fSpecifier->what=command;
				switch (command) {
					case B_NAME_SPECIFIER:
						if (!fSpecifier->HasString("name"))
							fSpecifier->AddString("name","");
						break;
					case B_ID_SPECIFIER:
						if (!fSpecifier->HasInt32("id"))
							fSpecifier->AddInt32("id",0);
						break;
					case B_RANGE_SPECIFIER:
					case B_REVERSE_RANGE_SPECIFIER:
						if (!fSpecifier->HasInt32("range"))
							fSpecifier->AddInt32("range",0);
					case B_INDEX_SPECIFIER:
					case B_REVERSE_INDEX_SPECIFIER:
						if (!fSpecifier->HasInt32("index"))
							fSpecifier->AddInt32("index",0);
						break;
				}
				SendUpdate("what",0);
			}
			break;
/*
		case K_UPDATE_ITEM:
			Invoke();
			break;
*/
		default:
			inherited::MessageReceived(msg);
	}
}

//void wSpecifierEditor::SetSpecifier(BMessage *msg,int32 what)
void wSpecifierEditor::UpdateCurrentRecord(bool selectionChanged)
{
	bool enabled=false;
	fSpecifier=Table()->CurrentDataRecord();
	int32 what=fSpecifier->what;
	if (fSpecifier!=NULL) {
		fSpecMenu->ItemAt(0)->SetMarked(true);
		for (int32 i=0;specifier_table[i]>-1;i++) {
			if (specifier_table[i]==what) {
				(fSpecMenu->ItemAt(i))->SetMarked(true);
				enabled=true;
				break;
			}
		}
	} else {
		fSpecMenu->ItemAt(0)->SetMarked(true);
	}
}

// ----------------------------------------------------------------

wMessageSpecifierViewer::wMessageSpecifierViewer(BRect frame, const char *name,wTable *table,uint32 resizingMode, uint32 flags):
inherited(frame,name,B_SINGLE_SELECTION_LIST,resizingMode,flags),
wDBWidget(table)
{
}

void wMessageSpecifierViewer::DisplayMessageSpecifier(BMessage *msg)
{
	MakeEmpty();
	// -----
	if (!msg->HasSpecifiers()) return;
	int32 index,what,count;
	type_code type;
	BMessage specifier;
	char *property;
	BListItem *item;
	msg->GetInfo("specifiers",&type,&count);
	/* pb: if message was not delivered, popspecifier won't work...
	while (msg->GetCurrentSpecifier(&index,&specifier,&what,&property)==B_OK&&count-->0) {
		AddItem(item=new wSpecifierItem(&specifier,what,property));
		msg->PopSpecifier();
	}
	*/
	while (msg->FindMessage("specifiers",--count,&specifier)==B_OK) {
		AddItem(item=new wSpecifierItem(&specifier,specifier.what,specifier.FindString("property")));
	}
}

void wMessageSpecifierViewer::MessageReceived(BMessage *message)
{
	switch (message->what) {
		case K_UPDATE:
			UpdateCurrentRecord(false);
			break;
		default:
			inherited::MessageReceived(message);
	}
}

void wMessageSpecifierViewer::SelectionChanged()
{
	wSpecifierItem *item=(wSpecifierItem *)ItemAt(CurrentSelection(0));
	SelectRecord(item);
}

BMessage *wMessageSpecifierViewer::CurrentSpecifier()
{
	BListItem *item=ItemAt(CurrentSelection(0));
	if (item!=NULL) {
		return ((wSpecifierItem *)item)->Specifier();
	} else return NULL; 
}

void wMessageSpecifierViewer::UpdateCurrentRecord(bool selectionChanged)
{
	if (!selectionChanged) {
		BListItem *item=ItemAt(CurrentSelection(0));
		if (item!=NULL) {	
			InvalidateItem(CurrentSelection(0));
		}
	}
}

BMessage *wMessageSpecifierViewer::AddSpecifier(BMessage *specifier)
{
	if (specifier==NULL) {
		specifier=new BMessage(B_DIRECT_SPECIFIER);
		specifier->AddString("property","???");
	}
	AddItem(new wSpecifierItem(specifier,B_DIRECT_SPECIFIER,"???"));
	Select(CountItems()-1);
	return specifier;
}

void wMessageSpecifierViewer::RemoveSpecifier()
{
	BListItem *item=RemoveItem(CurrentSelection(0));
	if (item!=NULL)
		delete item;
}

// ----------------------------------------------------------------

wMessageSpecifierEditor::wMessageSpecifierEditor(BRect frame,const char *name,uint32 resizingMode, uint32 flags):
inherited(frame,name,resizingMode,flags)
{
	fSpecifierTable=new wTable("specifier_table");
	fMessageSpecifierViewer=new wMessageSpecifierViewer(BRect(0,0,100,100),"w_specifiers_viewer",fSpecifierTable);
	fSpecifierEditor=new wSpecifierEditor(BRect(0,0,100,50),"w_specifier_editor",fSpecifierTable);
}

wMessageSpecifierEditor::~wMessageSpecifierEditor()
{
	fSpecifierTable->CloseTable();
}

void wMessageSpecifierEditor::AttachedToWindow(bool first_time)
{
	if (first_time) {
		// viewer ---
		// buttons ---
		BButton *button;
		KPackView *v=new KPackView(BRect(0,0,10,10),"buttons_panel",B_FOLLOW_ALL_SIDES,0,B_FANCY_BORDER);
		v->PackChild(button=new BButton(BRect(0,0,50,20),"b_add","Add",new BMessage(K_ADD)),K_LEFT,K_TOP);
		button->SetTarget(this);
		v->PackChild(button=new BButton(BRect(0,0,50,20),"b_remove","Remove",new BMessage(K_REMOVE)),K_LEFT,K_TOP);
		button->SetTarget(this);
		//PackChild(v,K_BOTTOM,K_BOTTOM,K_FILL_X);
		// pack ---
		PackChild(add_scrollview(fMessageSpecifierViewer,false,true),K_LEFT,K_LEFT,K_FILL_Y);
		Split();
		PackChild(v,K_BOTTOM,K_LEFT,K_FILL_X);
		PackChild(fSpecifierEditor,K_FULL,K_TOP,K_FILL_BOTH);
		// --
		fMessageSpecifierViewer->SetSelectionMessage(new BMessage(K_SELECT));
		fMessageSpecifierViewer->SetTarget(this);
		//fSpecifierEditor->SetMessage(new BMessage(K_UPDATE_ITEM));
		//fSpecifierEditor->SetTarget(this);
	}
}

void wMessageSpecifierEditor::MessageReceived(BMessage *msg)
{
	void *source;
	switch (msg->what) {
		case K_ADD:
			fMessageSpecifierViewer->AddSpecifier();
			break;
		case K_REMOVE:
			fMessageSpecifierViewer->RemoveSpecifier();
			break;
		default:
			inherited::MessageReceived(msg);
	}
}

void wMessageSpecifierEditor::SetMessage(BMessage *msg)
{
	if (msg!=NULL) {
		fMessageSpecifierViewer->DisplayMessageSpecifier(msg);
	}
}

void wMessageSpecifierEditor::MimicMessage(BMessage *msg)
{
	wSpecifierItem *item;
	for (int32 i=fMessageSpecifierViewer->CountItems()-1;i>=0;--i) {
		item=(wSpecifierItem *)fMessageSpecifierViewer->ItemAt(i);
		msg->AddSpecifier(item->Specifier());
	}
	msg->ReplaceInt32("_cur_specifier_",fMessageSpecifierViewer->CountItems()-1);
}
