#ifndef SIV_WINDOW_H
#define SIV_WINDOW_H

#include <InterfaceKit.h>
#include <StorageKit.h>

class KBitmapView : public BView 
{
public:
	
	KBitmapView(BRect rect,const char *name,uint32 resizingMode, uint32 flags);
	~KBitmapView();
	
	void SetZoomFactor(float factor);
	float ZoomFactor();
	
	virtual void Draw(BRect update);
	
	void SetTo(BBitmap *bitmap);
	BBitmap *ReleaseBitmap();
	
	virtual void GetPreferredSize(float *width, float *height);
	
	virtual void AttachedToWindow(void);
	
	virtual void FrameResized(float width, float height);
	virtual void ScrollTo(BPoint origin);
	virtual void ScrollTo(int32 value,orientation posture);
	
	virtual void MouseDown(BPoint where);
	virtual void MouseUp(BPoint where);
	virtual void MouseMoved(BPoint point, uint32 transit, const BMessage *message);
	
private:

	void clean_up();

	void compute_actual_bounds();
	void resize_to_fit();

	typedef BView inherited;
	
	BBitmap	*fBitmap,*fDblBitmap;
	BPoint 	fMouseOffset;
	bool	fScrolling;
	BRect	fScrollRect;
	
	float	fZoomFactor;
	BRect	fActualBounds;
	bool 	fLockScrollTo;
};

class SIVWindow : public BWindow 
{
public:
	SIVWindow(BRect frame,const char *title);
	
	status_t LoadImage(const char *path);
	void UpdateDisplay();
	void ForceMouseWithin();
	
	virtual void MessageReceived(BMessage *message);
	virtual void DispatchMessage(BMessage *message, BHandler *target);

	virtual BHandler *ResolveSpecifier(BMessage *message, int32 index, BMessage *specifier, int32 what, const char *property);
	virtual bool QuitRequested();
	
	virtual void FrameMoved(BPoint screenPoint);
	
	virtual void InvokePopUp(BPoint where);
	
	static filter_result filter
		(BMessage *message, BHandler **target,BMessageFilter *messageFilter);
	static filter_result spy_filter
		(BMessage *message, BHandler **target,BMessageFilter *messageFilter);
	
private:
	typedef	BWindow inherited;
	
	void Init();
	
	BPoint fMouseOffset;
	
	KBitmapView	*fMainView;
	BScrollView *fScrollView;
	
	BPath	fImagePath;
};

#endif
